
/****************************************************************************** 
 _______  _______ _________ _          _______  _______  ______   _______ 
(       )(  ___  )\__   __/( (    /|  (  ____ \(  ___  )(  __  \ (  ____ \
| () () || (   ) |   ) (   |  \  ( |  | (    \/| (   ) || (  \  )| (    \/
| || || || (___) |   | |   |   \ | |  | |      | |   | || |   ) || (__    
| |(_)| ||  ___  |   | |   | (\ \) |  | |      | |   | || |   | ||  __)   
| |   | || (   ) |   | |   | | \   |  | |      | |   | || |   ) || (      
| )   ( || )   ( |___) (___| )  \  |  | (____/\| (___) || (__/  )| (____/\
|/     \||/     \|\_______/|/    )_)  (_______/(_______)(______/ (_______/
                                                                          
******************************************************************************/


#include <p32xxxx.h>

#include "system.h"
#include <QueueList.h>
#include <Time.h>
#include <Wire.h>
#include <MCP794xxRTC.h>    // a basic MCP794xx library
#include <EEPROM.h>

#include <SPI.h>
#include <Ethernet.h>  
#include <Flash.h>
#include <SD.h>
#include <DSPI.h>
#include <TinyWebServer.h>
#include <XBee.h>
#include <Sensors.h>


extern MCP794xxRTC RTC;
extern XBee xbee;
extern UserSensorsList  UsrDefSensors;
extern int new_cosm_update;
extern char buffer_iot[IOT];
extern char RxPacketBuf[RxBufSize];
extern char buffer_status[160];


String localtime;
uint32_t previousMillis;


// The LED attached in PIN 13 on an Arduino board.
const int ledPin = 13;

const int HW_SS2A = 53;

// The initial state of the LED
int ledState = LOW;

// variables for the chipSolar board
int V5_status;
int charger;
int fault;

void setLedEnabled(boolean state) {
  ledState = state;
  digitalWrite(ledPin, ledState);
}

inline boolean getLedState() { return ledState; }

boolean file_handler(TinyWebServer& web_server);
boolean blink_led_handler(TinyWebServer& web_server);
boolean led_status_handler(TinyWebServer& web_server);
boolean index_handler(TinyWebServer& web_server);

TinyWebServer::PathHandler handlers[] = {
  // Work around Arduino's IDE preprocessor bug in handling /* inside
  // strings.
  //
  // `put_handler' is defined in TinyWebServer
  {"/", TinyWebServer::GET, &index_handler },
  {"/submitform" "*", TinyWebServer::GET, &TinyWebFormHandler::form_handler },
  {"/upload/" "*", TinyWebServer::PUT, &TinyWebPutHandler::put_handler },
  {"/chipkit-cgi" "*", TinyWebServer::GET, &TinyWebFormHandler::chipkit_cgi_handler },
  {"/ledstatus" "*", TinyWebServer::GET, &led_status_handler },
  {"/listfiles/", TinyWebServer::GET, &listfileshandler },
  {"/initop/", TinyWebServer::GET, &initophandler },
  {"/remove/" "*", TinyWebServer::GET, &removefilehandler },
  {"/" "*", TinyWebServer::GET, &file_handler }, // This should be always the last one, actualy is the position number 7, see TinyWebServer.cpp:285 if need to change.
  {NULL},
};

const char* headers[] = {
  "Content-Length",
  NULL
};

TinyWebServer web = TinyWebServer(handlers, headers);

TinyWebFormHandler::FormList form_handler_list[] = {   
  {"available", NULL},
  {"datetime", &datetime_handler},
  {"addphone", &add_ph_handler},
  {"addmail", &add_mail_handler},
  {"login", &login_handler},  
  {"addxbeedev", &add_sensor_handler},
  {NULL},                                    
};

TinyWebFormHandler::cgiList cgi_handler_list[] = {
  {"available", NULL},
  {"localtime", &show_datetime},
  {"numtab", &number_table},
  {"mailtab", &email_table},
  {"sensortab", &sensor_table},
  {"logtab", &log_table},
  {"cmlog", &cmlog},
  {"lasttab", &lasttab},
  {"cosmst", &cosmtest},
  {"cosmcfg", &cosmcfg},
  {"testtab", &testtab},
  {"e403c8806", &e403c8806},
  {"e405ca391", &e405ca391},
  {NULL},                                    
};

boolean has_filesystem = true;
Sd2Card card;
SdVolume volume;
SdFile root;
SdFile file;

static uint8_t mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };

// Don't forget to modify the IP to an available one on your home network
byte ip[] = { 192, 168, 1, 105 };

int ready2requestSMS = 1;
int ready2requestEmail = 0;
int EmailRetry = 1;


void send_file_name(TinyWebServer& web_server, const char* filename) {
  if (!filename) {
    web_server.send_error_code(404);
    web_server << F("Could not parse URL");
  } else {
    TinyWebServer::MimeType mime_type
      = TinyWebServer::get_mime_type_from_filename(filename);
    web_server.send_error_code(200);
    web_server.send_content_type(mime_type);
    web_server.end_headers();
    Serial.println("gonna try to open file");
    Serial.println(filename);
    //SD.exists((char *)filename);
    if (file.open(&root, filename, O_READ)) {
      Serial << F("Read file "); Serial.println(filename);
      web_server.send_file(file);
      file.close();
    } else {
      web_server << F("Could not find file: ") << filename << "\n";
    }
  }
}

boolean file_handler(TinyWebServer& web_server) {
  char* filename = TinyWebServer::get_file_from_path(web_server.get_path());
  send_file_name(web_server, filename);
  free(filename);
  return true;
}


boolean removefilehandler(TinyWebServer& web_server){
  char* filename = TinyWebServer::get_file_from_path(web_server.get_path());
  Serial.println(filename);
  if (filename) {
    if(file.remove(&root, filename)){
      Serial.println("file removed");
    }else{
      Serial.println("file remove fail!");
    }
  }
  web_server.send_error_code(200);
  web_server.send_content_type("text/plain");
  web_server.end_headers();  
  free(filename);
}

boolean listfileshandler(TinyWebServer& web_server) {
  web_server.send_error_code(200);
  web_server.send_content_type("text/plain");
  web_server.end_headers();
  web_server << F("<html><body><h1>List files OK</h1></body></html>\n");  
  Serial.println();
  Serial.println("\nFiles found on the card (name, date and size in bytes): ");
  // list all files in the card with date and size
  root.ls(LS_R | LS_DATE | LS_SIZE);  
  Serial.println();
}

boolean initophandler(TinyWebServer& web_server) {
  web_server.send_error_code(200);
  web_server.send_content_type("text/plain");
  web_server.end_headers();  
  web_server << F("<html><body><h1>Fatory reset done!</h1></body></html>\n");  
  Serial.println("Erasing SRAM memory");
  InitProgState();        // Initialize the Memory State program - this should only be done once....
  Serial.println(" Success");
}

boolean blink_led_handler(TinyWebServer& web_server) {
  web_server.send_error_code(200);
  web_server.send_content_type("text/plain");
  web_server.end_headers();
  // Reverse the state of the LED.
  setLedEnabled(!getLedState());
  Client& client = web_server.get_client();
  if (client.available()) {
    char ch = (char)client.read();
    if (ch == '0') {
      setLedEnabled(false);
    } else if (ch == '1') {
      setLedEnabled(true);
    }
  }
}

boolean led_status_handler(TinyWebServer& web_server) {
  web_server.send_error_code(200);
  web_server.send_content_type("text/plain");
  web_server.end_headers();
  Client& client = web_server.get_client();
  client.println(getLedState(), DEC);
}

boolean index_handler(TinyWebServer& web_server) {
//  if(web_server.session_is_active()) {
    send_file_name(web_server, "INDEX.HTM");
//  }else{
//    send_file_name(web_server, "login.htm");
//  }
  return true;
}

boolean add_ph_handler(TinyWebServer& web_server) {
  Serial.println(web_server.get_path());
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<html><body><h1>Form was submitted</h1></body></html>\n");
  char *phone = web_server.get_var_from_uri(web_server.get_path(), "addnum");
  Serial.print("Adding phone number ");
  Serial.print(phone);
  Serial.println(" to the list.");
  String temph = String(phone);
  SaveEntryMem(temph);
  free(phone);
  return true;
}

boolean add_mail_handler(TinyWebServer& web_server) {
  Serial.println(web_server.get_path());
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<html><body><h1>Form was submitted</h1></body></html>\n");  
  char *email = web_server.get_var_from_uri(web_server.get_path(), "email");
  Serial.print("Adding email address ");
  Serial.print(email);
  Serial.println(" to the list.");
  String tempmail = String(email);
  SaveEmailMem(tempmail);
  free(email);
  return true;
}

boolean add_sensor_handler(TinyWebServer& web_server) {
  Serial.println(web_server.get_path());
  web_server.send_error_code(200);
  web_server.end_headers();  
  char *msb = web_server.get_var_from_uri(web_server.get_path(), "msb");
  char *lsb = web_server.get_var_from_uri(web_server.get_path(), "lsb");
  Serial.println("Adding xbee device");
  Serial.print("msb address is ");
  Serial.println(msb);
  Serial.print("lsb address is ");
  Serial.println(lsb);
  uint32_t n, k;
  sscanf(msb,"%x",&n);
  sscanf(lsb,"%x",&k);
  Serial.println("The MSB is ");
  Serial.println(n, HEX);
  Serial.println("The LSB is ");
  Serial.println(k, HEX);  
  free(msb);
  free(lsb);
  return true;
}
  
boolean datetime_handler(TinyWebServer& web_server){
  tmElements_t t;
  
  Serial.println();
  Serial.println("datetime_handler");
  Serial.println(web_server.get_path());
  web_server.send_error_code(200);
  web_server.end_headers();
  char *day = web_server.get_var_from_uri(web_server.get_path(), "day");  
  char *month = web_server.get_var_from_uri(web_server.get_path(), "month");
  char *year = web_server.get_var_from_uri(web_server.get_path(), "year");
  char *hours = web_server.get_var_from_uri(web_server.get_path(), "hours");
  char *minutes = web_server.get_var_from_uri(web_server.get_path(), "minutes");
  char *Wday = web_server.get_var_from_uri(web_server.get_path(), "weekday");
  t.Second = 0;
  Serial.print("Day is ");
  Serial.println(day); 
  t.Day = (uint8_t)atoi(day);
  Serial.print("Month is ");
  Serial.println(month);  
  t.Month = (uint8_t)atoi(month);
  Serial.print("Year is ");
  Serial.println(year);   
  t.Year = CalendarYrToTm((uint8_t)atoi(year));
  Serial.print("Hours is ");
  Serial.println(hours);
  t.Hour = (uint8_t)atoi(hours);
  Serial.print("Minutes is ");
  Serial.println(minutes);
  t.Minute = (uint8_t)atoi(minutes);
  Serial.print("WeekDay is ");
  t.Wday = (uint8_t)atoi(Wday) + 1;
  Serial.println(t.Wday, DEC);
  
  RTC.set(makeTime(t));
  setSyncInterval(SYNC_RTC);
  setSyncProvider(RTC.get);   // the function to get the time from the RTC
  
  if(timeStatus()!= timeSet) 
     Serial.println("Unable to sync with the RTC");
  else
     Serial.println("RTC has set the system time");     
  Serial.println();
  free(day);
  free(month);
  free(year);
  free(hours);
  free(minutes);
  free(Wday);
  return true;
}


boolean login_handler(TinyWebServer& web_server) {
  char *username = web_server.get_var_from_uri(web_server.get_path(), "user");
  char *password = web_server.get_var_from_uri(web_server.get_path(), "pass");
  if((strcmp(username, "ge_oseo") == 0) && (strcmp(password, "cryomonitor") == 0)) {
    web_server.activate_session();  
    web_server.send_redirect("main.htm");
  }else{
    web_server.send_redirect("login.htm");
  }
  free(username);
  free(password);
  return true;
}



void file_uploader_handler(TinyWebServer& web_server,
			   TinyWebPutHandler::PutAction action,
			   char* buffer, int size_var) {
  static uint32_t start_time;
  static uint32_t total_size;

  switch (action) {
  case TinyWebPutHandler::START:
    start_time = millis();
    total_size = 0;
    if (!file.isOpen()) {
      // File is not opened, create it. First obtain the desired name
      // from the request path.
      char* fname = web_server.get_file_from_path(web_server.get_path());
      if (fname) {
	Serial << F("Creating ") << fname << "\n";
	if(!file.open(&root, fname, O_CREAT | O_WRITE | O_TRUNC)){
          Serial << F("Could not be created") << "\n";
          if(file.remove(&root, fname)){        // file.remove first call to file.open, it fails which already happen try to delete the file.
            Serial << F("File already exist, deleting file") << "\n";
          }          
        }
	free(fname);
      }
    }
  break;

  case TinyWebPutHandler::WRITE:
    if (file.isOpen()) {
      file.write(buffer, size_var);
      total_size += size_var;
    }
  break;

  case TinyWebPutHandler::END:
    if (file.isOpen()) {
      file.sync();
      Serial << F("Wrote ") << file.fileSize() << F(" bytes in ")
  	   << millis() - start_time << F(" millis (received ")
             << total_size << F(" bytes)\n");
      file.close();
    }
  }
}


// ***********************
// CGI handlers
// ***********************

boolean number_table(TinyWebServer& web_server) {
  Serial.println("number_table method");
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<table id=""ContactNumber"" width=""175px"" border=""1"">");   
  web_server << F("<tr><td> Phone number </td></tr>");
  char buf1[] = "<tr><td>";
  char buf2[11];              // the phone number container
  char buf3[] = "</td></tr>";
  char tempPhone[12];
//  String tempPhone;
 
  memset(tempPhone, 0, 12);
  Serial.println();
  Serial.println("Number of Entries...");
  Serial.println(getNumberOfEntries(), DEC);
  Serial.println(); 
  
  for(int j=0; j<getNumberOfEntries(); j++){      
      web_server << F(buf1);
      QueryEntry(tempPhone, j);
      //tempPhone.toCharArray(buf2, tempPhone.length()+1);
      web_server << F(tempPhone);
      web_server << F(buf3);
  }
  web_server << F("</table>\n");
}

boolean email_table(TinyWebServer& web_server) {
  Serial.println("email_table method");
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<table id=""ContactMail"" width=""175px"" border=""1"">");   
  web_server << F("<tr><td> e-mail address </td></tr>");
  char buf1[] = "<tr><td>";
  char buf2[40];              // mhanuel@ieee.org
  char buf3[] = "</td></tr>";
  String tempMail;
  
  Serial.println();
  Serial.println("Number of Entries...");
  Serial.println(getNumberOfEntriesMail(), DEC);
  Serial.println();
  
  for(int j=0; j<getNumberOfEntriesMail(); j++){      
      web_server << F(buf1);
      tempMail = QueryEmailEntry(j);
      tempMail.toCharArray(buf2, tempMail.length()+1);
      web_server << F(buf2);
      web_server << F(buf3);
  }
  web_server << F("</table>\n");
}

boolean sensor_table(TinyWebServer& web_server) {
  char tsensor[500];
  
  memset(tsensor, 0, 500);
  Serial.println("sensor_table method");
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<table id=""SensorTable"" width=""400px"" border=""1"">");
  web_server << F("<tr><th>Address MSB</th>");
  web_server << F("<th>Address LSB</th>");
  web_server << F("<th>Type</th>");
  web_server << F("<th>Parent</th></tr>");
  UsrDefSensors.printHTMLtable(tsensor);
  Serial.println("");
  Serial.println(tsensor);
  Serial.println("");
  web_server << F(tsensor);
  web_server << F("</table>\n");
  Serial.println("done sensor_table");
}

boolean log_table(TinyWebServer& web_server) {
  char tsensor[500];
  
  memset(tsensor, 0, 500);
  Serial.println("log_table method");
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<table id=""LogTable"" width=""400px"" border=""1"">");
  web_server << F("<tr><th>Address MSB</th>");
  web_server << F("<th>Address LSB</th>");
  web_server << F("<th>File Download</th></tr>");
//  web_server << F("<tr><td>0x0013a200</td><td>0x405CA391</td><td><a href=""http://192.168.1.105/dig1.csv"" target=""_blank"">Log File</a></td></tr>");
  UsrDefSensors.printLogTable(tsensor);
  Serial.println("");
  Serial.println(tsensor);
  Serial.println("");
  web_server << F(tsensor);
  web_server << F("</table>\n");
  Serial.println("done log_table");
}


boolean show_datetime(TinyWebServer& web_server) {
  Serial.println("show local time method");
  web_server.send_error_code(200);
  web_server.end_headers();
  String t = getSysDateTime();
  char time[20];
  web_server << F("<b>");
  t.toCharArray(time, t.length()+1);
  web_server << F(time);
  web_server << F("</b>\n");
}

boolean cmlog(TinyWebServer& web_server) {
  Serial.println("CryMonitor Log file cgi download");
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<b>""Home Controller Log File: ""<b>");
  web_server << F("<a href=""http://192.168.1.105/CMLOG.TXT"" target=""_blank"">Download</a>\n");
  Serial.println("done hcm_log");
}

boolean lasttab(TinyWebServer& web_server) {
  String lastactstr;
  char lastactivity[200];
  
  memset(lastactivity, 0, 200);
  Serial.println("CryMonitor Last Activity");
  web_server.send_error_code(200);
  web_server.end_headers();
  lastactstr = buffer_status;
  int j;
  char temp[50];
  
  for(int k=0; k<200; k++){
    j = lastactstr.indexOf("\n\r");
    if(j < 0)
      break;
    strcat(lastactivity, "<b>");
    lastactstr.toCharArray(temp, j+1);
    strcat(lastactivity, temp);
    strcat(lastactivity, "</b><br>");
    lastactstr = lastactstr.substring(j+2);
    memset(temp, 0, 50);
  };
  Serial.println("Format to web");
  Serial.println(lastactivity);  
  web_server << F(lastactivity);
  Serial.println("done lasttab");
}

boolean cosmtest(TinyWebServer& web_server) {
 
  Serial.println("CryMonitor cosmtest");
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<b>CGI Under Construction!</b>\n");
  Serial.println("done cosmtest");
}

boolean cosmcfg(TinyWebServer& web_server) {
 
  Serial.println("CryMonitor cosmcfg");
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<b>Domain: </b>");  
  web_server << F(COSM_DOMAIN);
  web_server << F("<br>");  
  web_server << F("<b>Feed Id: </b>");  
  web_server << F(SHARE_FEED_ID_STRING);
  web_server << F("<br>");
  Serial.println("done cosmcfg");
}
 
boolean testtab(TinyWebServer& web_server){
  char tsensor[500];
  
  memset(tsensor, 0, 500);
  Serial.println("CryMonitor testtab"); 
  
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<script>");
  web_server << F("$(\"#e405ca391\").click(function (event)\n{\nevent.preventDefault();\nalert(\"Force a Sensor to Send Current Sample.\");$(\"#force\").load(\"chipkit-cgi/e405ca391\");\n});\n");
  web_server << F("$(\"#e403c8806\").click(function (event)\n{\nevent.preventDefault();\nalert(\"Force a Sensor to Send Current Sample.\");$(\"#force\").load(\"chipkit-cgi/e403c8806\");\n});");
  web_server << F("</script>");
  web_server << F("<table id=""SensorTable"" width=""400px"" border=""1"">");
  web_server << F("<tr><th>Address MSB</th>");
  web_server << F("<th>Address LSB</th>");
  web_server << F("<th>Type</th>");
  web_server << F("<th>Action</th></tr>");
  UsrDefSensors.printTESTtable(tsensor);
  Serial.println("");
  Serial.println(tsensor);
  Serial.println("");
  web_server << F(tsensor);
  web_server << F("</table>\n");
  Serial.println("done test_table"); 
}
  
boolean e405ca391(TinyWebServer& web_server){
  Serial.println("CryMonitor end device address 405ca391 force sample method");
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<b> Test Compressor Sensor done</b>");
  uint32_t msb,lsb;
  sscanf ("13a200","%x",&msb);
  sscanf ("405ca391","%x",&lsb);
  ForcesSampleFnc(msb, lsb);
}

boolean e403c8806(TinyWebServer& web_server){
  Serial.println("CryMonitor end device address 403c8806 force sample method");
  web_server.send_error_code(200);
  web_server.end_headers();
  web_server << F("<b> Test Sensor 403c8806 done</b>");
  uint32_t msb,lsb;
  sscanf ("13a200","%x",&msb);
  sscanf ("403c8806","%x",&lsb);
  Serial.print("MSB: ");
  Serial.println(msb, HEX);
  Serial.print("LSB: ");
  Serial.println(lsb, HEX);  
  ForcesSampleFnc(msb, lsb);  
}


// ***********************
// Local Functions
// ***********************


String getSysDateTime(void){
  time_t t = now();
  String datetime;
  
  datetime += String(hour(t));
  datetime += ":";
  int m = minute(t);
  if(m < 10)
    datetime += "0";
  datetime += String(m);  
  datetime += ":";  
  int sec = second(t);
  if(sec < 10)
    datetime += "0";
  datetime += String(sec);
  datetime += " ";
  datetime += String(day(t));
  datetime += "/";
  datetime += String(month(t));
  datetime += "/";
  datetime += String(year(t));
  
  return datetime;
}


// ***********************
// Update Cosm
// ***********************

void UpdateCosm(void){
   
    #if defined __SIM340__                                   // for SIM340 AT+CIPSHUT can not be issue if the state is IP INITIAL
    if(modemRequest("AT+CIPSTATUS", 1, 1, "STATE: IP INITIAL") < 0){
      modemRequest("AT+CIPSHUT", 1, 1, "SHUT OK");
    }
    // at this point IP Status should move to IP INITIAL
    if(modemRequest("AT+CIPSTATUS", 1, 1, "STATE: IP INITIAL")){    // connect only is IP Status is IP INITIAL 
    #else                                                    // for SIM900 this command can be issue at any time...
    if(modemRequest("AT+CIPSHUT", 1, 1, "SHUT OK") == 1){    // Shut down any GPRS if Open or in TCP CLOSED state or any other state not allowed for a AT+CIPSTART command
    #endif
      if(modemConnect(COSM_DOMAIN, PORT)){          // this option is better because IP could change. An additional command must be issue inside modemConnect function
        Serial.println("********************************************************");
        Serial.println(buffer_iot);
        Serial.println("********************************************************");
        modemSend(buffer_iot, strlen(buffer_iot));
      }
    }  
}


// ***********************
// Main Log
// ***********************

void MainLogUpdate(char logbuffer[]){
    
  Serial.println("Updating Device Main Log");
  if(file.open(&root, "CMLOG.TXT", O_CREAT | O_APPEND | O_WRITE)){
    Serial.println(logbuffer);
    file.write(logbuffer, strlen(logbuffer));
    file.sync();
    file.close();        
  }
   #if defined __debug_cmd__
   else{
     Serial.println("Could not open File CMLOG.TXT");
   }
   #endif
}


// ***********************
// SetUp
// ***********************


void setup() {
  // Init some varible
  memset(buffer_status, 0, 160);          // Buffer that hold the status of Netwrk Sensors.
  
  Serial.begin(9600);      // use for console messages
  Serial.println("Home Controller Starting ...");
  
  // Init the PM module status lines
  pinMode(LT4411_STAT, INPUT);
  pinMode(LTC3652_CHRG, INPUT);
  pinMode(LTC3652_FAULT, INPUT);
  // Init Modem Power Control Line  
  pinMode(MODEM_PWR, OUTPUT);
  digitalWrite(MODEM_PWR, LOW);         // a High Level applied at the PWRKEY Pin of Modem (remember the NPN transistor)
  
  #if defined __XBeeNetwork__
  xbee.begin(9600);
  // Uses Serial Port 2 to allow monitoring output on Serial Port 0.
  Serial2.begin(9600);
  xbee.setSerial(Serial2);
  delay(500);
  #endif
  

  
  #if defined __SIM340__
  Serial.println("Modem Serial Port Baudrate 115200");  // This is used for GPRS Modem communication.
  Serial1.begin(115200);
  #else
  Serial.println("Modem Serial Port Baudrate 57600");   // This is used for GPRS Modem communication.
  Serial1.begin(57600);
  #endif
  
 
  Serial.println("De-select the SD card");
  pinMode(HW_SS2A, OUTPUT);         // default SPI SS signal.
  digitalWrite(HW_SS2A, HIGH);      // de-select the SD card on chipWireless board.

//  pinMode(ledPin, OUTPUT);
//  setLedEnabled(false);

  #if defined __SD_CARD__
  // initialize the SD card
  Serial << F("Setting up SD card...\n");
  pinMode(10, OUTPUT); // set the SS pin as an output (necessary!)
  digitalWrite(10, HIGH); // but turn off the W5100 chip!
  
  if (!card.init(SPI_HALF_SPEED, HW_SS2A)) {        // This should be called before Ethernet begin
    Serial << F("card failed\n");
    has_filesystem = false;
  }
  // initialize a FAT volume
  if (!volume.init(&card)) {
    Serial << F("vol.init failed!\n");
    has_filesystem = false;
  }
  if (!root.openRoot(&volume)) {
    Serial << F("openRoot failed");
    has_filesystem = false;
  }else{
    Serial << F("Setting Root Handler");
    UsrDefSensors.SetRootHandler(&root);  
  }
  #endif


  #if defined __Web_Server__
  Serial << F("Setting up the Ethernet card...\n");
  Ethernet.begin(mac, ip);
  
  // Start the web server.
  Serial << F("Starting Web server...\n");
  web.begin();
  
  if (has_filesystem) {
    // Assign our function to `upload_handler_fn'.
    TinyWebPutHandler::put_handler_fn = file_uploader_handler;
    // Assign the form funstions to "form_handler_list"
    TinyWebFormHandler::form_handler_list_ptr = form_handler_list;
    // Assign the cgi funstions to "cgi_handler_list"
    TinyWebFormHandler::cgi_handler_list_ptr = cgi_handler_list;
  }
  #endif
  

  // init the RTCC Sync provider, values are updated via HTTP
  #if defined __RTCC__
  time_t t = RTC.get();
  setTime(t);
  setSyncInterval(SYNC_RTC);
  setSyncProvider(RTC.get);   // the function to get the time from the RTC
  #endif
  
  
//  Serial.println("Memory content");
//  uint8_t j = 0; 
//  for(int k=0; k<64; k++){
//    RTC.read_byte(k, j);                  // read current entry number
//    Serial.print(j, HEX);
//    Serial.print("  ");
//    if(k == 7 || k == 15 || k==23 || k==31 || k==39 || k==47 || k==55)
//      Serial.println();
//  }
  
  // used to initialize the operation memory.
  Serial.println();
  previousMillis = millis();
  
//  // Charger Status
//  V5_status = digitalRead(LT4411_STAT);  
//  charger = digitalRead(LTC3652_CHRG);  
//  fault = digitalRead(LTC3652_FAULT);     
//  Serial.println();
//  Serial.println("Charger Status");
//  Serial.print("Ideal Diode Status Pin is: ");
//  if(V5_status)
//    Serial.println("OFF");                    // High impedance state
//  else
//    Serial.println("ON");                     // Pulled Low
//  Serial.print("Charger Pin Value is: ");
//  if(charger)
//    Serial.println("OFF");                    // High impedance state
//  else
//    Serial.println("ON");                     // Pulled Low
//  Serial.print("Fault Pin Value is: ");
//  if(fault)
//    Serial.println("OFF");                    // High impedance state
//  else
//    Serial.println("ON");                     // Pulled Low   
//  Serial.println();
//  
//  

  memset(RxPacketBuf, 0, 1000);          // Init RxPacket Buffer
  
  #if defined __GPRS_compile__  
  Serial.println("Modem Initialization...");
  while(!modemIsInitialized()){
    modemInit("GSM850_PCS_MODE");        // some basic initialization and test communication with modem
    delay(500);                         // if we should retray wait some time
    if(!SimCardIsPresent()){
      Serial.println("a>SimCard not present");
      break;
    }
  }
  
  Serial.println("Check GRPRS Network");
  while(!modemIsRegistered()){
    modemCheckNetwork();
    delay(1000);
  }
  Serial.println("Initializing GPRS Modem for Movilnet Operator");
  if(!modemInitGPRS("IP", "int.movilnet.com.ve")){
    while(1);                          // Error while defining PDP context or other GPRS parameter.
  }
  Serial.println("Activate Modem GPRS Interface");
  if(!modemActivateGPRS("int.movilnet.com.ve", 1))  
    Serial.println("Could not activate GPRS");
  Serial.println("setting SMS mode");
  modemSMS_init();  
  
  #endif  // end of __GPRS_compile__ 
  
  
  // Initialize the XBee Network
  #if defined __XBeeNetwork__
  Serial.println("XBee Network Setup...");
  InitSensorNetwork(); 
  #endif
  
  #if defined  __Watch_Dog__
  attachCoreTimerService(MyCallback);
  WDTCONbits.ON = 1;      //Enable the WDT 
  #endif
  
  Serial << F("Ready Init.\n");
  
}



void loop() {
  // RTCC Time Handler
  if(millis() - previousMillis >= 1000){
      localtime = getSysDateTime();
      previousMillis = millis();
      #if defined __GPRS_compile__  
      if(ready2requestSMS == 0){
        Serial.println("Send SMS");
        SensorNetworkStatus(localtime);                                  // get status of wireless network
        ready2requestSMS = 1;
        ready2requestEmail = 1;
      }
      if(ready2requestEmail){
        if(send_Mail() == -1){
          if(EmailRetry){
            TurnOffModem();
            modemInit("GSM850_PCS_MODE");
            EmailRetry = 0;
            Serial.println("After Power Cycling still cannot send mail");
          }else{
            ready2requestEmail = 0;
            EmailRetry = 1;
            EnableSMSIndications();
            checkUnreadSMS();
          }
        }else{
          ready2requestEmail = 0;
          EmailRetry = 1;
          EnableSMSIndications();
          checkUnreadSMS();
        }
      }
      #endif
  }
   
  #if defined __Web_Server__ 
  // Web Server Handler
  if (has_filesystem) {
    web.process();
  }
  #endif
  
  // XBee Network Handler
  #if defined __XBeeNetwork__
  XBeeNetworkLoop();
  #endif
  // COSM Update Handler
  
  #if defined __GPRS_compile__
  if(new_cosm_update > 0){
    DisableSMSIndications();
    Serial.println("Enter UpdateCosm Method");
    UpdateCosm();
//    ready2requestSMS = 0;  // just for test.... if the above line is commented... else this is done in ReceiveData
    new_cosm_update = 0;
  }
  // GPRS Receive Data Handler
  ReceiveData();
  ProcessSMS();
  cmd_SMS();
  #endif

}


uint32_t MyCallback(uint32_t currentTime) {
  WDTCONbits.WDTCLR = 1;
  return (currentTime + CORE_TICK_RATE/10);
}

