/******************************************************************************

               _______ _________ _______   _____   _______  _______ 
              (  ____ \\__   __/(       ) / ___ \ (  __   )(  __   )
              | (    \/   ) (   | () () |( (   ) )| (  )  || (  )  |
              | (_____    | |   | || || |( (___) || | /   || | /   |
              (_____  )   | |   | |(_)| | \____  || (/ /) || (/ /) |
                    ) |   | |   | |   | |      ) ||   / | ||   / | |
              /\____) |___) (___| )   ( |/\____) )|  (__) ||  (__) |
              \_______)\_______/|/     \|\______/ (_______)(_______)


**********************       SIM900 Modem Library     *************************
******************************************************************************/

// includes
#include "system.h"
#include <QueueList.h>

// defintions 
// modem status
#define STATE_NONE 0
#define STATE_ON 1
#define STATE_INITIALIZED 2
#define STATE_REGISTERED 4
#define STATE_ACTIVE	8
//#define STATE_CONNECTED 16
#define STATE_SYNC 32
#define STATE_NO_SIMCARD 64    



//  Variables
unsigned int state;
char modemRespGlobal[500];                                // hold response to commands, AT+CMGL is probably one command that need a large bufer depending on SMS number
char* modemRespGlobalPtr = &modemRespGlobal[0];
char LocalIP[16];
int startof, endof, rx_len;
QueueList <String> n_msg_queue;
QueueList <String> n_cmd_queue;
QueueList <String> n_phone_queue;


char RxPacketBuf[RxBufSize];
char *RxPacketPtr = &RxPacketBuf[0];

extern char buffer_status[160];
extern uint8_t clear_buffer;
extern int ready2requestSMS;

int mailtimeout = 0;
/***************************************************************************** 
// Modem Status Functions
/***************************************************************************** 

/***************************************************************************** 
 * Function: modemIsSync
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

unsigned int modemIsSync(void){
  return (state & STATE_SYNC);
}

/***************************************************************************** 
 * Function: modemIsInitialized
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

unsigned int modemIsInitialized(void) {
  return (state & STATE_INITIALIZED);
}

/*****************************************************************************
 * Function: modemIsRegistered
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

unsigned int modemIsRegistered(void){
  return (state & STATE_REGISTERED);
}

/***************************************************************************** 
 * Function: modemIsActivated
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/


unsigned int modemIsActivated(void){
  return (state & STATE_ACTIVE);
}

/***************************************************************************** 
 * Function: SimCardPresent
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/


unsigned int SimCardIsPresent(void){
  return (state & STATE_NO_SIMCARD);
}

/***************************************************************************** 
 * Function: modemIsConnected
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

//unsigned int modemIsConnected(void){
//  return (state & STATE_CONNECTED);
//}

/***************************************************************************** 
                    _________ _       __________________
                    \__   __/( (    /|\__   __/\__   __/
                       ) (   |  \  ( |   ) (      ) (   
                       | |   |   \ | |   | |      | |   
                       | |   | (\ \) |   | |      | |   
                       | |   | | \   |   | |      | |   
                    ___) (___| )  \  |___) (___   | |   
                    \_______/|/    )_)\_______/   )_(   

// Modem Initialization and Configuration Functions
*****************************************************************************/


/*****************************************************************************
 * Function: TurnOnModem()
 *
 * Precondition: None
 *
 * Overview: Turn On Modem
 *           
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

void TurnOnModem(void){
	
	#if defined __debug__
	Serial.println("BEGIN TURN ON SIM900 SEQUENCE ...!");
	#endif	
	
	digitalWrite(MODEM_PWR, HIGH);  		// inverted pulse to Power On Modem (remember it get inverted by the NPN transistor)
        #if defined __SIM340__
	delay(3000);	                                // The pulse duration t > 2s for SIM340 and t > 1s for SIM900
        #else
	delay(1500);    				//	HIGH	/--   t   --/    HIGH 
	#endif						//_____________	   	     ______________	
							//	       |____________|
                                                        //                   LOW
	digitalWrite(MODEM_PWR, LOW);			// Return Normal State	

	#if defined __debug__
	Serial.println("MODEM SHOULD BE ON!");
	#endif			

        delay(2500);                                    // Wait for serial Port to become active	

}

/*****************************************************************************
 * Function: TurnOnModem()
 *
 * Precondition: None
 *
 * Overview: Turn On Modem
 *           
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

int TurnOffModem(void){
  int res;
  res = modemRequest("AT+CPOWD=1", 3, 1, "NORMAL POWER DOWN");	            // Turn Off Modem command, wait more than one second. 
  if(res){
    return(1);              // modem serial connection OK 
  }else{
    return(0);              // there is a problem with serial connection.
  } 
}




/*****************************************************************************
 * Function: modemSync
 *
 * Precondition: Serial Port Initialized
 *
 * Overview: Sends "A" character to modem and wait 3 seconds for the modem
 *           to synchronized baud rate before sending the AT command. 
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

int modemSync(void){
  #if defined __SIM900__  
  Serial1.write("A");       // Send "A" to synchronized baud rate as suggested in the datasheet.
  #endif
  delay(4000);              // wait 3 seconds.
  return(serial_OK());
}

int serial_OK(void){
  int res;
  res = modemRequest("AT", 1, 1, "OK");
  Serial.println("res variable is: ");
  Serial.println(res, DEC);
  if(res > 0){
    Serial.println("AT command OK");
    return(1);              // modem serial connection OK 
  }else{
    Serial.println("AT command Fail");
    return(0);              // there is a problem with serial connection.
  } 
}
  
/*****************************************************************************
 * Function: modemRequest
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

int modemRequest(char *command, unsigned char timeout, unsigned char check, char str2cmp[]){

    int count = 0;               
                                 
    Serial1.flush();                // discard anything already in the buffer to allow a new command get processed
                                  
    #if defined __debug_cmd__      // output debugging commands send to modem.
    Serial.print("a>");             // Arduino Prompt              
    Serial.println(command);        // print the command that's being send to the modem
    #endif                    
    while(*command != '\0'){  
    	Serial1.write(*command);    // Send command to modem
	command++;
    }
    Serial1.write('\r');    
    
    count =  getsTimeOut(timeout, check, str2cmp);    // check the modem response.
    return count;
}

/*****************************************************************************
 * Function: getsTimeOut
 *
 * Precondition: 
 *
 * Overview: 
 * 
 *
 * Input: 
 *
 * Output: 
 *
 *****************************************************************************/


int getsTimeOut(unsigned int timeout, unsigned char check, char str2cmp[]){

    unsigned long ltimeout = 1000*timeout;
    char temp;
    char* tmp_buf;
    char* end_buf;
    int str_len, rx_len;
//    String modemResponse;
    int cmp_str;
    #if defined __debug_cmd__
//    String modemResponseCpy;
    int j;
    #endif
    
    memset(modemRespGlobal, 0, 1000);
    modemRespGlobalPtr = &modemRespGlobal[0];
    
    delay(ltimeout);

    str_len = Serial1.available();        // DON'T TRY TO DO THE FOLLWING!  if(Serial1.available() > 0) it just don't work let's have some time to see the serial library for the reason.
    if(str_len > 0){
      while(Serial1.available()){
        temp = Serial1.read();
        *modemRespGlobalPtr = temp;
        modemRespGlobalPtr++;
      }
      
      #if defined __debug_cmd__
      if(strlen(modemRespGlobal) > 0){
        #if defined __format_output__
        tmp_buf = modemRespGlobal;
        for(int j=0;;){
          end_buf = strstr(tmp_buf,"\r\n");
          if(end_buf == NULL)
            break;
          if(tmp_buf == end_buf){
            tmp_buf += 2; 
            continue;  
          }
          Serial.print("m>");
          for(int k=0;;){
            if(tmp_buf == end_buf){
              tmp_buf += 2; 
              Serial.println("");
              break;
            }
            Serial.print(*tmp_buf);
            tmp_buf++;
          }
        }
        #elif defined __no_format__
        Serial.print(modemRespGlobal);
        #endif
        
//        #if defined __format_output__
//        while(1){
//          j = modemResponseCpy.indexOf("\r\n");
//          if(j<modemResponseCpy.length()){
//            if(j>0){
//              Serial.print("m>");
//              Serial.println(modemResponseCpy.substring(0, j));     // print just until \ char is encountered
//            }
//            modemResponseCpy = modemResponseCpy.substring(j+2);      // include the following char after \r\n
//          }else{
//            break;              
//          }
//        }
//        #elif defined __no_format__
//        Serial.print(modemResponseCpy);
//        #endif
      }
      #endif
      
      if(check){
        if(strstr(modemRespGlobal, str2cmp) != NULL){
            str_len = 1;                    // return 1 if strings match     
            #if defined __debug__
            Serial.println("a>Got a response that match! ");       // if debug defined go verbose
            #endif             
        }else if(strstr(modemRespGlobal, "ERROR") != NULL){
            str_len = -2;                                        // return -2 if ERROR
            #if defined __debug__
            Serial.println("a>Got an ERROR in response: ");        // if debug defined go verbose
            Serial.print("a>");
            Serial.println(modemRespGlobal);
            #endif       
        }else{
            str_len = -1;                                        // return -1 if strings doesn't match.
            #if defined __debug__
            Serial.println("a>Got an Unknown response");         // if debug defined go verbose
            Serial.print("a>");
            Serial.println(modemRespGlobal);
            #endif
        }       
      }
      
      
//      if(check){
//        cmp_str = modemResponse.indexOf(str2cmp);
//        if(cmp_str == -1){
//          cmp_str = modemResponse.indexOf("ERROR");
//          if(cmp_str >= 0){
//            str_len = -2;                                        // return -2 if ERROR
//            #if defined __debug__
//            Serial.println("a>Got an ERROR in response: ");        // if debug defined go verbose
//            Serial.print("a>");
//            Serial.println(modemResponse);
//            #endif
//          }else{
//            str_len = -1;                                        // return -1 if strings doesn't match.
//            #if defined __debug__
//            Serial.println("a>Got an Unknown response");         // if debug defined go verbose
//            Serial.print("a>");
//            Serial.println(modemResponse);
//            #endif
//          }
//        }else{
//          cmp_str = modemResponse.indexOf(str2cmp);
//          if(cmp_str >= 0){
//            str_len = 1;                    // return 1 if strings match     
//            #if defined __debug__
//            Serial.println("a>Got a response that match! ");       // if debug defined go verbose
//            #endif 
//          }else{
//            str_len = -1;
//            #if defined __debug__
//            Serial.println("a>Not expected response");       // if debug defined go verbose
//            #endif             
//          }
//        }  
//      }
      
    }else{
      #if defined __debug_cmd__
      if(ltimeout > 0){
        Serial.println("TimeOut");
        str_len = -3;
      }
      #endif    
    }
    return(str_len);                                             // return the string match result or zero if no serial data available (communication problem)
}

/*****************************************************************************
 * Function: getCmpFromLast
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: timeout and string to compare with
 *
 * Output: 1 if there is a match, -1 otherwise
 *
 *****************************************************************************/
 
int getCmpFromLast(unsigned int timeout, unsigned char check, char str2cmp[]){
 
  unsigned long ltimeout = 1000*timeout;
  int str_len = 0;
//  String modemResponse;
  char temp;
  int cmp_str = 0;
  
  delay(ltimeout);                      // do a delay
  
  str_len = Serial1.available();        // check whether ther is some caracters in receive buffer.
  
  if(str_len>0){
    while(str_len--){
      temp = Serial1.read();            // extract the caracters
      *modemRespGlobalPtr = temp;
      modemRespGlobalPtr++;      
    }
  }
  str_len = strlen(modemRespGlobal);
  
  if(check){                                 // check if global variable have the string to compare inside...
    if(strstr(modemRespGlobal, str2cmp) != NULL){
      str_len = 1;             // return 1 if a match.  
    }else{
      str_len = -1;            // return -1 if there is no match between the received string and the one given to compare with  
    }
  } 
  
  return(str_len);             // if str_len = 0 no new caracter were received only if check was not done. 
}


/*****************************************************************************
 * Function: modemInit
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

unsigned char modemInit(String OpBAND){
  
  int res = 0;
  String temp_str;
  char buffer[40];
    
  if(!serial_OK()){            // return 0 if modem do not replay the AT command.
    #if defined __debug_cmd__
    Serial.println("Modem is Off, init Turn On sequence");
    #endif
    TurnOnModem();             // if not succesful then turn on modem 
    if(modemSync() == 0){      // send command to sync        
      return(0);  
    }  
  }
  // If execution get to this point it means modem is On with 
  // Serial Port Active and seirla communication OK 
  res = modemRequest("AT+CMEE=2", 1, 1, "OK");	// enable error result code and use verbose values
  if(res < 0){
      return(0);
  }
  res = modemRequest("ATE0", 1, 1, "OK");	               // Echo Mode Off
  if(res < 0){
      return(0);
  }
  res = modemRequest("AT+CSMINS?", 1, 1, "+CSMINS: 0,1");      // SIM inserted status report
  if(res < 0){
      state &= ~STATE_NO_SIMCARD;
      #if defined __debug__
      Serial.println("SIM CARD not inserted!");
      #endif
      return(0);          // SIM CARD not inserted
  }
  state |= STATE_NO_SIMCARD;
  temp_str = "AT+CBAND=\"";
  temp_str += OpBAND;
  temp_str += "\"";
  temp_str.toCharArray(buffer, temp_str.length()+1);
  res = modemRequest(buffer, 1, 1, "OK");                      // Set Mobile Operation Band
  #if defined __debug_cmd__L2
  Serial.print("res variable result of cmd : ");
  Serial.print(buffer);
  Serial.print("is ;");
  Serial.println(res);
  Serial.print("modem response global variable is: ");
  Serial.println(modemRespGlobal);
  #endif           
  if(res < 0){
      res = getCmpFromLast(1, 1, "+PDP: DEACT");               // if we got this message the gprs is release by 
      if(res == 1){                                            // the network but we still need to issue AT+CIPSHUT command
        #if defined __debug__
        Serial.println("Shutdown GPRS");
        #endif          
        res = modemRequest("AT+CIPSHUT", 1, 0, "");           // don't check the response!
      }
      return(0);
  }

  state |= STATE_INITIALIZED;
  return(1);  
}

/*****************************************************************************
 * Function: modemCheckNetwork
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

void modemCheckNetwork(void){
  char *p;	
  int res;
  
  res = modemRequest("AT+CREG?", 2, 1, "+CREG: 0,1");
  if(res > 0){
    state |= STATE_REGISTERED;
    #if defined __debug__
    Serial.println("Network registered, home network");
    #endif
  }else{
    state &= ~STATE_REGISTERED;
  }
}

/*****************************************************************************
 * Function: modemInitGPRS
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

int modemInitGPRS(String PDP_type, String APN){
  int res;
  String temp_str;
  char buffer[50];
  
  res = modemRequest("AT+CIPSPRT=1", 1, 1, "OK");                       // Prompt Of ‘>’ When Sending Data and shows "send ok" when sending is succesful
  #if defined __SIM340__
  res += modemRequest("AT+CDNSORIP=0", 1, 1, "OK");                     // Connect with IP Address (if =1 connet with nameserver)
  #endif
  res += modemRequest("AT+CIPSRIP=0", 1, 1, "OK");                      // Do not Show IP Address And Port Of Sender When Receive Data.
  res += modemRequest("AT+CIPHEAD=1", 1, 1, "OK");                      // Add An IP Head When Receiving Data
  temp_str = "AT+CGDCONT=1,\"";                                         // Make a string of this form AT+CGDCONT=1,"PDP_type","APN"
  temp_str += PDP_type;
  temp_str += "\",\"";
  temp_str += APN;
  temp_str += "\"";
  temp_str.toCharArray(buffer, temp_str.length()+1);
  res += modemRequest(buffer, 1, 1, "OK");	                        // Define PDP Context
  res += modemRequest("AT+CIPSCONT", 1, 1, "OK");                       // Save TCP/IP Application Context
  
  #if defined __SIM340__
  if(res == 6){
  #else
  if(res == 5){
  #endif
    #if defined __debug__
    Serial.println("GPRS Initialization sucessful");
    #endif
    return(1);      // all commands were completed corrected
  }else{
    return(0);      // all commands were completed corrected
  }
}

/*****************************************************************************
 * Function: modemActivateGPRS
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/


int modemActivateGPRS(String APN, unsigned char multi){
  int res, res2;
  String temp_str;
  char buffer[40];
  unsigned char valid = 0;
  int n;
  uint32_t previousMillis;
  
  // Need to be in IP INITIAL state, if not just SHUT DOWN CONTEXT.
  do{
    res2 = modemRequest("AT+CIPSTATUS", 1, 1, "STATE: IP INITIAL");      // Keep checking for IP INITIAL STATE until get there
    if(res2 < 0){                      // if not in STATE: IP INITIAL state
      res = modemRequest("AT+CIPSHUT", 1, 1, "SHUT OK");    // Shut down GPRS
      if(res <= 0){    
        return(0);                     // need an OK of this command if not just exit
      }
    }else if(res2 == 0){
      return(0);        //need to receive something else exit and abort (comm problem with modem)
    }
  }while(res2 < 0);
  res += modemRequest("AT+CIPMODE=0", 1, 1, "OK");                       // Normal Mode
  
  //************************************************************************
  // THE FOLLOWING IS ONLY NEEDED IN MULTI-IP STATE ("AT+CSTT, AT+CIICR, AT+CIFSR")
  //************************************************************************
  // At this point modem is in STATE: IP INITIAL state which is needed for the following command (AT+CSTT)
  // now we need to start the context
  // issue AT+CSTT command
  if(multi){
    temp_str = "AT+CSTT=\"";
    temp_str += APN;                            // This will make the temp_str look like this AT+CSTT="APN","","" (i.e. AT+CSTT="int.movilnet.com.ve","","")
    temp_str += "\",\"\",\"\"";                 // with the last to "" corresponding to USER NAME and PASSWORD
    temp_str.toCharArray(buffer, temp_str.length()+1);
    do{
      res = modemRequest(buffer, 1, 1, "OK");
      res2 = modemRequest("AT+CIPSTATUS", 1, 1, "STATE: IP START");
      if(res2 < 0){
        res = modemRequest("AT+CIPSHUT", 1, 1, "SHUT OK");
        if(res <= 0){
          return(0);
        }
      }else if(res2 == 0){
        return(0);          //need to receive something else exit and abort (comm problem with modem)
      }
    }while(res2 < 0);
    // At this point modem is in STATE: IP START state
    // Need to activate the GPRS or bring Up Wireless conection - THIS MAY TAKE A WHILE
    // issue AT+CIICR command
    
    res = modemRequest("AT+CIICR", 1, 0, "");
    previousMillis = millis();
    do{
      res = getCmpFromLast(1, 1, "OK");
      if(res>0){
        Serial.println("");
        Serial.println("AT+CIICR response OK");
        break;
      }
      if(millis() - previousMillis >= 10000){
        Serial.println("");
        Serial.println("AT+CIICR TimeOut");
        return(0); 
      }
      Serial.print(" . ");
    }while(res == 0);    
    
    // let's check that GPRS is UP and active
    res = modemRequest("AT+CIPSTATUS", 1, 1, "STATE: IP GPRSACT");
    if(res < 0){
      #if defined __debug__
      Serial.println("GPRS Connection is not active!");
      #endif    
      return(0);      // if connection is not active (IP GPRSACT) then just leave... let's see if this is true
    }	
    #if defined __debug__
    Serial.println("Get the GPRS assigned IP address ...");
    #endif  
    // issue AT+CIFSR command
    res = modemRequest("AT+CIFSR", 2, 0, "");                 // the command response don't have an OK... and we are not matching so the only response is 0 for nothing at serial
    do{
      res = getCmpFromLast(1, 0, "");                          // wait 1 more second and just return to the loop with the number of caracters received.
      if(res > 0){            // port receive buffer or >0 for something that we need to check ... IP or ERROR    
        if(strstr(modemRespGlobal, "ERROR") != NULL){
          #if defined __debug_cmd__
          Serial.println("AT+CIFSR output an error");
          #endif
          return(0);
        }else{
          #if defined __debug_cmd__
          Serial.println("Need to Check the IP address");
          #endif
          valid = VerifyIPAddress(modemRespGlobal);    // get rid of the \r\n that comes from modem output...
          if (!valid){
            #if defined __debug__
            Serial.println("Not a valid IP address");
            #endif
            return(0);
          }//else just leave with the res > 0 value to exit do while loop
        }
      }else if(res == 0){
         #if defined __debug_cmd__
         Serial.println("still waiting for the local IP address command reply");
         #endif
      }
    }while(res == 0);            // WAIT UNTIL THE MODEM RESPOND TO COMMAND  
  }
  
  state |= STATE_ACTIVE;
  return(1);
}


/*****************************************************************************

          _________ _______  _______   _________ _______ 
          \__   __/(  ____ \(  ____ )  \__   __/(  ____ )
             ) (   | (    \/| (    )|     ) (   | (    )|
             | |   | |      | (____)|     | |   | (____)|
             | |   | |      |  _____)     | |   |  _____)
             | |   | |      | (           | |   | (      
             | |   | (____/\| )        ___) (___| )      
             )_(   (_______/|/         \_______/|/       
                                              
                                              
*****************************************************************************/

/*****************************************************************************
 * Function: modemConnect
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/


int modemConnect(String IP, int port){          // IP or hostname allowed
  int res;
  int n;
  String temp_str; 
  char buffer[50];
	
  #if defined __SIM900__
  res = modemRequest("AT+CIPQSEND=0", 1, 1, "OK");    // Normal Mode, SEND OK response when modem receive TCP data
  if(res < 0){
    #if defined __debug__
    Serial.println("Normal Mode could not be set");
    #endif    
    return(0);      // if connection is not active (IP GPRSACT) then just leave... let's see if this is true
  }
  #endif
  #if defined __SIM340__
  res = modemRequest("AT+CDNSORIP=1", 1, 1, "OK");
  if(res < 0){
    #if defined __debug__
    Serial.println("Connect with domain name command fail");
    #endif    
    return(0);      // if connection is not active (IP GPRSACT) then just leave... let's see if this is true
  }
  #endif  
  #if defined __debug_cmd__
  Serial.println("trying to connect");
  #endif
  temp_str = "AT+CIPSTART=\"TCP\",\"";
  temp_str += IP;
  temp_str += "\",";
  temp_str += port;
  temp_str += "\r";
  temp_str.toCharArray(buffer, temp_str.length()+1);
  res = modemRequest(buffer, 1, 1, "OK");
  #if defined __debug_cmd__L2
  Serial.print("pass1 modemConnect() res value = ");
  Serial.println(res);
  #endif
  if(res){
    // we get here if format is right!
    for(n=0; n<30; n++){
      res = getCmpFromLast(1, 1, "CONNECT OK");
      #if defined __debug_cmd__L2_
      Serial.print("pass2 modemConnect() res value = ");
      Serial.println(res);
      #endif
      if(res == 1){
          res = modemRequest("AT+CIPSTATUS", 1, 1, "CONNECT OK");          // A strange error was logged... see Issue_001.txt
          if(res == 1){
            #if defined __debug_cmd__
            Serial.print("chipKIT now connected to IP: ");
            Serial.println(IP);
            #endif
            return(1);                               // If connected just exit with code 1
          }
      }                                            // else, try to see if connect have failed.
      res = getCmpFromLast(0, 1, "CONNECT FAIL");
      #if defined __debug_cmd__L2
      Serial.print("pass3 modemConnect() res value = ");
      Serial.println(res);      
      #endif
      if(res == 1){
        #if defined __debug_cmd__
        res = modemRequest("AT+CIPSTATUS", 1, 0, "");        //Just print the response of this command for debugging
        Serial.print("chipKIT fail to connect to IP: ");
        Serial.println(IP);
        #endif
        return(0);                                  // If connect fail just exit with code 0
      }
      res = getCmpFromLast(0, 1, "ALREADY CONNECT");
      #if defined __debug_cmd__L2
      Serial.print("pass4 modemConnect() res value = ");
      Serial.println(res);      
      #endif      
      if(res == 1){                              // Already connected
        #if defined __debug_cmd__
        res = modemRequest("AT+CIPSTATUS", 1, 0, "");        //Just print the response of this command for debugging
        Serial.print("chipKIT already connected to IP: ");
        Serial.println(IP);
        #endif
        return(0);                                  // If connect fail just exit with code 0
      }      
      res = getCmpFromLast(0, 1, "+CME ERROR");
      #if defined __debug_cmd__L2
      Serial.print("pass5 modemConnect() res value = ");
      Serial.println(res);      
      #endif      
      if(res == 1){                                  // Error related to ME functionality...
        #if defined __debug_cmd__
        res = modemRequest("AT+CIPSTATUS", 1, 0, "");        //Just print the response of this command for debugging
        Serial.print("Error with Modem: ");
        #endif
        return(0);                                  // If connect fail just exit with code 0
      }       
    }
  }
  
  return(0);
}

/*****************************************************************************
 * Function: modemClose
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

int modemClose(void){
  int res;
  
  res = modemRequest("AT+CIPCLOSE", 1, 1, "CLOSE OK");
  if(res == 1){
    #if defined __debug_cmd__
    Serial.println("close connection ok");
    #endif
  }else{
    #if defined __debug_cmd__
    Serial.println("close connection fail");
    #endif
  }
  return(1);
}




/*****************************************************************************
 * Function: modemSend
 *
 * Precondition: None.
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

int modemSend(char *data, unsigned int len){
  
  int res;
  char buffer[30];
  int timeout, keepalive;
  char send_cmd[18];
  char length_ptr[5];
  
  memset(send_cmd, 0, 18);
  strcpy(send_cmd, "AT+CIPSEND=");
  itoa(len, length_ptr, 10);
  strcat(send_cmd, length_ptr);
  Serial.println("Send Command is: ");
  Serial.println(send_cmd);
//  send_cmd += len;
//  send_cmd.toCharArray(buffer, send_cmd.length()+1);
  res = modemRequest(send_cmd, 1, 1, ">");
  #if defined __debug_cmd__L2
  Serial.print("pass1 modemSend() res value = ");
  Serial.println(res);
  #endif
  if(res){
    while(len){
      Serial1.print(*data++);
      len--;
    }
    while(1){
      if(timeout > 1500){                      // 1500 * 10ms ~ aprox 15 seconds
        #if defined __debug_cmd__
        Serial.println("modemSend TimeOut");
        #endif
        Serial1.flush();
        res = 0;
        break;                                                  
      }
      timeout++;
      delay(10);
      res =  getCmpFromLast(0, 1, "SEND OK");               // If SEND OK (this function support only Normal Mode...)
      if(res == 1){
        #if defined __debug_cmd__
        Serial.println("data succesfully sent");
        #endif
        //res = 1;
        break;
      }
      res =  getCmpFromLast(0, 1, "+CME ERROR");             // if ERROR related to ME functionality.
      if(res == 1){
        #if defined __debug_cmd__
        Serial.println("there was an ME error while sending data");
        #endif
        res = 0;
        break;
      }
      res =  getCmpFromLast(0, 1, "SEND FAIL");
      if(res == 1){
        #if defined __debug_cmd__
        Serial.print("fail to send data");
        #endif
        res = 0;
        break;
      }
      if(res <= 0){
        #if defined __debug_cmd__
        keepalive++;
        if(keepalive > 100){
          keepalive = 0;
          Serial.print(".");
        }
        #endif
      }
    }
  }
  #if defined __debug_cmd__L2
  Serial.println("Exit modemSend()");
  #endif
  return(res);
}

/*****************************************************************************
 * Function: ReceiveData
 *
 * Precondition: 
 *
 * Overview: 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/


int ReceiveData(void){            // Only prints messages that are received after client sends data and have the form +IPD,xxx:DATA
  String temp;
  
  char msg_index[3];            
  char tempch;
  char len_buf[4];
  char *findptr1, *findptr2;
  char datalen[5];
  int  ipdlen;
  
  if(Serial1.available()){                                       // this is a minimun length without data and two digits for length of rx data... we can adjust this later...
    tempch = Serial1.read();
    *RxPacketPtr++ = tempch;
    if(strlen(RxPacketBuf) < 4)
      return(0);
    
    // Check for TCP
    findptr1 = strstr(RxPacketBuf, "+IPD");
    if(findptr1 != NULL){        
      findptr2 = strstr(findptr1, ":");
      if(findptr2 != NULL){
        Serial.println("colon(:) found");  
        int m = findptr2 - (findptr1+4);
        Serial.println("number of chars: ");
        Serial.println(m, DEC);
        if(m > 0 && m < 4){
          memset(datalen, 0, 5);
          strncpy(datalen, (findptr1+4), (unsigned int)m);          // +IPD543:
          datalen[(unsigned int)m] = '\0';
          Serial.println("data length is: ");
          Serial.println(datalen);
          ipdlen = atoi(datalen);
          if(ipdlen == 0)
            return(0);
          Serial.print("a> pass2, ReceiveData() length is = ");
          Serial.println(ipdlen);  
          while(ipdlen){
            tempch = Serial1.read();
            *RxPacketPtr++ = tempch;
            ipdlen--;
          }
          
          // Do something with the data
          
          Serial.println("a> pass3, reception complete");
          Serial.println("******************************");
          Serial.println("a> pass1:");        
          findptr2++;                            // point to first char in data
          Serial.print(findptr2);
          Serial.println("******************************"); 
          if(strstr(findptr2, "HTTP/1.1 200 OK")){  
            Serial.println("Cosm update OK");
            ready2requestSMS = 0;
          }
    
          // End doing something with the data
              
          RxPacketPtr = &RxPacketBuf[0];
          memset(RxPacketBuf, 0, RxBufSize);          // Init RxPacket Buffer  
          return(1);  
        }else{
          Serial.println("an error occur during +IPD reception");
          return(0);
        }
      }
      return(0);
    }// end of check for TCP Data 
    
    // check for SMS  
    findptr1 = strstr(RxPacketBuf, "+CMTI: \"SM\",");            // if a new meesage indication have arrive I got +CMTI: "SM", in response
    if(findptr1 != NULL){                                        // let's process the message...  
      while(1){
        tempch = Serial1.read();
        *RxPacketPtr++ = tempch;            
        if(tempch == '\n')
          break;
      }   
      findptr2 = strstr(findptr1 + 12, "\r\n");                  // length of +CMTI: "SM",
      if(findptr2 != NULL){
        memset(msg_index, 0, 3);
        strncpy(msg_index, (findptr1 + 12), findptr2 - (findptr1 + 12));     // include anything after this ->(+CMTI: "SM",) until "\r\n" is found. that Should be the message number.
        temp = String(msg_index);          
        Serial.print("New SMS, msg index: ");
        Serial.println(temp);    
        n_msg_queue.push(temp);                // push the index of received message for processing.
        RxPacketPtr = &RxPacketBuf[0];
        memset(RxPacketBuf, 0, RxBufSize);          // Init RxPacket Buffer  
        return(1);              
      }
  
    }// end of SMS reception...
    
  }// End of Serial reception
  
  // should check for RING, NO CARRIER, etc
  return(0);
}


// ****************************
// SMS FUNTIONALITY STUFF
// ****************************

/*****************************************************************************
 * Function: modemSMS_init
 *
 * Precondition: None.
 *
 * Overview: Configure SMS for text mode.
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
 
int modemSMS_init(void){
  int res; 
  
  res = modemRequest("AT+CMGF=1", 1, 1, "OK");
  if(res){
    return(1);              // modem serial connection OK 
  }else{
    return(0);              // there is a problem with serial connection.
  }
}

/*****************************************************************************
 * Function: ProcessSMS
 *
 * Precondition: None.
 *
 * Overview: Process any SMS using a queue
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

void ProcessSMS(void){                       // this function must bealled periodically.

  if(!n_msg_queue.isEmpty()){                // check if there is a new SMS
  
//    String query = "AT+CMGR=";                 // command to get a given SMS from memory (SIM card memory)
    String nSMS, nPhone, bSMS;                 // SMS number, sender's phone number, body SMS (data).
    int res;
    char tempbuf[3];
    char* tmp_ptr;
    char* end_ptr;
    char send_cmd[11];
    char length_ptr[5];
    
    nSMS = n_msg_queue.pop();                // pop a message from queue (the index was push)
    nSMS.toCharArray(tempbuf, nSMS.length()+1);
    #if defined __debug_sms__
    Serial.print("processing SMS index#: ");
    Serial.println(nSMS);
    #endif
    

  
    memset(send_cmd, 0, 18);
    strcpy(send_cmd, "AT+CMGR=");
    strcat(send_cmd, tempbuf);
    Serial.println("Send Command is: ");
    Serial.println(send_cmd);
  
//    query += nSMS;
//    query.toCharArray(tempbuf, query.length()+1);
    res = modemRequest(send_cmd, 1, 1, "OK");                          // here is an example of response +CMGR: "REC UNREAD","04161234567",,"12/03/27,12:30:00-18"    
    if(res > 0){                                                      // chipkit deadline in my local time (Venezuela). f so we are in the contest...
       tmp_ptr = strstr(modemRespGlobal, "+CMGR: ");
       if(tmp_ptr != NULL){
         tmp_ptr = strstr(tmp_ptr, "\",\"");
         tmp_ptr += 3;  
         for(int j=0;j<12;j++){                    // 04166065383
           if(*tmp_ptr == '"')
             break;  
           nPhone += *tmp_ptr;
           tmp_ptr++;
         }
       }     
       #if defined __debug_sms__
       Serial.print("sender's phone number: ");
       Serial.println(nPhone);       
       #endif
       if(is_in_Dir(nPhone) >= 0){                                    // an entry zero is the first available... return negative if not found
         tmp_ptr = strstr(tmp_ptr, "\r\n");
         end_ptr = strstr(tmp_ptr, "\r\n\r\nOK");
         tmp_ptr += 2;
         if((tmp_ptr != NULL) && (end_ptr != NULL) && (tmp_ptr < end_ptr)){
           while(1){
             if(tmp_ptr == end_ptr)
               break;
             bSMS += *tmp_ptr;
             tmp_ptr++;
           }
         }else{
           Serial.println("Error getting Message Body");
         }
         #if defined __debug_sms__
         Serial.print("body message: ");
         Serial.println(bSMS);    
         Serial.println("Add body message to queue");   
         #endif
         n_phone_queue.push(nPhone);                                    // Push the sender number to allow further processing, like sending a response back
         n_cmd_queue.push(bSMS);                                        // Push the message body 
       }else{  //, number not authorized
         Serial.println("Number not authorized");
       }
       // in any case we need to erase the message from SIM
       memset(send_cmd, 0, 18);
       strcpy(send_cmd, "AT+CMGD=");
       strcat(send_cmd, tempbuf);                       // add message number in TE memory    
       res = modemRequest(send_cmd, 1, 1, "OK");
       #if defined __debug_sms__
       if(res){
         Serial.println("Message have been erased!");  
       }else{
         Serial.println("Problem while trying to erase message!");  
       }
       #endif
    }else{
       #if defined __debug_sms__
       Serial.println("There was an error retrieving the SMS");    
       #endif      
    }
  }
}

/*****************************************************************************
 * Function: checkUnreadSMS 
 *
 * Precondition: None.
 *
 * Overview: Check if there is any unread SMS, if so, place to queue.
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

int checkUnreadSMS(void){
  int res;
  String number;
  char* tmp_ptr;
  
  Serial.println("Check Unread SMS");
  res = modemRequest("AT+CMGL", 1, 1, "OK");                              // here is an example of response +CMGR: "REC UNREAD","04161234567",,"12/03/27,12:30:00-18"    
  if(res > 0){   
    int n=0;
    tmp_ptr = modemRespGlobal;
    for(int j=0;;){   
      tmp_ptr = strstr(tmp_ptr, "+CMGL: ") ;
      if(tmp_ptr != NULL){
        tmp_ptr += 7;
        for(int j=0; j<4; j++){
          if(*tmp_ptr == ',')
            break;
          number = *tmp_ptr++;
        }
        n_msg_queue.push(number); 
        Serial.println("Unread message number: ");
        Serial.println(number);
      }else
        break;
    }
  }

}

/*****************************************************************************
 * Function: EraseSMS
 *
 * Precondition: None.
 *
 * Overview: Erase all SMS from memory
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

void EraseSMS(void){
  char tempbuf[15];  
  int res = 0;
  char send_cmd[11];
  char msgnum[3];
  
  memset(tempbuf, 0, 15);
  for(int j=1; j<41; j++){
    memset(send_cmd, 0, 11);
    itoa(j, msgnum, 10);
    strcat(send_cmd, "AT+CMGD=");
    strcat(send_cmd, msgnum);
    res = modemRequest(send_cmd, 1, 1, "OK");    
    if(res > 0){
       Serial.print("Message #");
       Serial.println(j, DEC);
       Serial.print("was deleted");
    }else{
      break;  
    }
  }
}

/*****************************************************************************
 * Function: send_SMS_Status
 *
 * Precondition: None.
 *
 * Overview: Send SMS with Device Status
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/


int send_SMS_Status(void){
  String query, indexString;
  int index, cr;
  char tempbuf[200];  
  int nSMS = 0;
  uint8_t i=0;
  int res, nSendStatSMS = 0;
  char temp;
  char* tmp_ptr;
  char* end_ptr;
  char senderPh[12];
  
  memset(senderPh, 0, 12);
  i = getNumberOfEntries();
  if(i > 0){
    query = String("AT+CMGW=\"");                  // Write message to memory
    query += String("+5");                         // format number
    QueryEntry(senderPh, 0); 
    senderPh[0] = '8'; 
    query += String(senderPh);
    query += String("\"\r");   
//    Serial.println("Read Eeprom Site Name");
//    int value;
//    String tmpsite;
//    uint16_t l = 0x171;
//    for(int n=0; n < strlen(sitename); n++){
//      value = EEPROM.read(l);
//      tmpsite += (char)value;
//      l++;        
//    }  
//    query += tmpsite;
    query += sitename;
    query += String(buffer_status);                  //  Message Body
    query.toCharArray(tempbuf, query.length()+1);  
    *(tempbuf+strlen(tempbuf)) = 0x1A;               // Ctrl-Z
    Serial.println("SMS request pass 0");
    res = modemRequest(tempbuf, 3, 1, "OK");         // Wait a couple of seconds before checking response...
    if(res > 0){
      tmp_ptr = strstr(modemRespGlobal, ":");
      if(tmp_ptr != NULL){
        tmp_ptr += 2;
        end_ptr = strstr(tmp_ptr, "\r\n");
        for(int j=0; j<2; j++){
          if(tmp_ptr == end_ptr)
            break;
          indexString += *tmp_ptr++;
        }
      }else{
        Serial.println("modem request fail");
        return(-1);
      } 
      Serial.print("SMS save to position: ");
      Serial.println(indexString);
    }else{
      Serial.println("modem request fail");
      return(-1);
    }
    memset(tempbuf, 0, 200); 
    
    Serial.println("pass 2");
    query = String("AT+CMSS=");  
    query += indexString;
    query.toCharArray(tempbuf, query.length()+1);
    res = modemRequest(tempbuf, 5, 1, "OK");
    if(res > 0){
      Serial.println("Send SMS from Storage OK to phone address book position 0");
    }else{          
      Serial.println("Send not confirmed");
    }      
    if(i > 1){
      Serial.println("pass 3");
      for(int j=1; j<getNumberOfEntries(); j++){
        query = String("AT+CMSS=");            //AT+CMSS=3,"+584166065383"  
        query += indexString;
        query += String(",\"+5");
        QueryEntry(senderPh, j);
        senderPh[0] = '8';    
        query += String(senderPh);
        query += String("\""); 
        query.toCharArray(tempbuf, query.length()+1);
        res = modemRequest(tempbuf, 5, 1, "OK");
        if(res > 0){
          Serial.print("Send SMS from Storage OK to phone address book position ");
          Serial.println(j, DEC);
        }else{          
          Serial.println("Send not confirmed");
        }       
      }          
    }
    query = String("AT+CMGD=");
    query += indexString;
    query.toCharArray(tempbuf, query.length()+1);
    res = modemRequest(tempbuf, 1, 1, "OK");    
    if(res > 0){
      Serial.println("Delete Message from Memory");      
    }                                 
//    clear_buffer = 1;                                 // clear alarm message (SMS message)
  }else{
    Serial.println("There is no cell phone number to send alert message");
  }
}

/*****************************************************************************
 * Function: send_Mail
 *
 * Precondition: None.
 *
 * Overview: send_Email using SMTP account without SSL
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

int send_Mail(void){
  int i, rcptok;
  char temp[170];
  String tempMail;
  char buf[40];
  
  if(getNumberOfEntriesMail() <= 0){
    Serial.println("There is no E-mail address to send message"); 
    clear_buffer = 1;
    return(0);        
  }
      
  if(modemRequest("AT+CIPSTATUS", 1, 1, "STATE: IP INITIAL") < 0){
    modemRequest("AT+CIPSHUT", 1, 1, "SHUT OK");
  }      
  if(modemRequest("AT+CIPSTATUS", 1, 1, "STATE: IP INITIAL")){    
    if(modemConnect("mail.cantv.net", 25)){
      memset(RxPacketBuf, 0, RxBufSize);
      RxPacketPtr = &RxPacketBuf[0];
    }else{
      
      Serial.println("Could not connect with E-mail server");
      return(-1);  
    }
  }
  // CONNECT OK
  if(ReceiveDataBlock("ESMTP Exchange Server")){
    memset(temp, 0, 170);          
    strcat(temp, "HELO cantv.net\r\n\r");              // SEND 
    if(modemSend(temp, strlen(temp)) != 1){
      Serial.println("Could not send HELO ...");
      return(0);           // A PROBLEM OCCUR
    }
  }else{
    Serial.println("didn't receive server response 1");
    return(0);  
  }
  // SEND OK
  if(ReceiveDataBlock("pleased to meet you")){
    memset(temp, 0, 170);          
    strcat(temp, "MAIL FROM: <mariagonzh11@cantv.net>\r\n\r");              // SEND
    if(modemSend(temp, strlen(temp)) != 1){    
      Serial.println("Could not send MAIL FROM: ...");
      return(0);           // A PROBLEM OCCUR      
    }    
  }else{
    Serial.println("didn't receive server response 2");
    return(0);  
  }
//  
//  if(ReceiveDataBlock("Sender ok")){
//    memset(temp, 0, 170);          
//    strcat(temp, "RCPT TO: <mhanuel.usb@gmail.com>\r\n\r");              // SEND 
//    if(modemSend(temp, strlen(temp)) != 1){    
//      Serial.println("Could not send RCPT TO: ...");
//      return(0);           // A PROBLEM OCCUR      
//    }    
//  }else{
//    Serial.println("didn't receive server response 3");
//    return(0);  
//  }

  // SEND OK
  if(ReceiveDataBlock("Sender ok")){
    
    for(int j=0; j<getNumberOfEntriesMail(); j++){   
      memset(temp, 0, 170);   
      memset(buf, 0, 40);
      tempMail = QueryEmailEntry(j);
      tempMail.toCharArray(buf, tempMail.length()+1);
      strcat(temp, "RCPT TO: <");
      strcat(temp, buf);
      strcat(temp, ">\r\n\r");
      if(modemSend(temp, strlen(temp)) != 1){    
        Serial.print("Could not send RCPT TO: ");
        Serial.println(buf);
        return(0);                   // A problem occur trying to send data to modem...
      }else{
        if(ReceiveDataBlock("Recipient ok")){
          Serial.print("recipient ok mail address");
          Serial.print(buf);
          memset(temp, 0, 170);   
          rcptok = 1;
          continue;
        }else{
          Serial.print("send to recipient address ");
          Serial.print(buf);
          Serial.println("FAIL");
          continue;
        }
      }
    }
  }else{
    Serial.println("Sender was not accepte by server - ABORT");
    return(0);  
  }
  
  // SEND OK
  if(rcptok){
    memset(temp, 0, 170);          
    strcat(temp, "DATA\r\n\r");              // SEND 
    if(modemSend(temp, strlen(temp)) != 1){    
      Serial.println("Could not send DATA: ...");
      return(0);           // A PROBLEM OCCUR      
    }    
  }else{
    Serial.println("There is no recipient to send email - ABORT");
    return(0);  
  }  
  // SEND OK
  if(ReceiveDataBlock("Enter mail")){
    memset(temp, 0, 170);          
    String tmpbuf = String(buffer_status);
    tmpbuf.toCharArray(temp, tmpbuf.length()+1);    
    strcat(temp, "\r\n.\r\n\r");              // SEND 
    if(modemSend(temp, strlen(temp)) != 1){    
      Serial.println("Could not send DATA: ...");
      return(0);           // A PROBLEM OCCUR      
    }    
  }else{
    Serial.println("didn't receive server response 5");
    return(0);  
  }
  // SEND OK
  if(ReceiveDataBlock("Message accepted for delivery")){  
    modemRequest("AT+CIPCLOSE", 1, 1, "CLOSE OK");
    modemRequest("AT+CIPSHUT", 1, 1, "SHUT OK");
    Serial.println("Message delivery OK");
    clear_buffer = 1;
  }
  
}



int ReceiveDataBlock(char findbuffer[]){

  int  rxbuflen;
  char *findptr1, *findptr2;
  char datalen[4];
  uint8_t mailtimeout = 0;
  int  ipdlen;
  
  while(1){
    Serial.print(".");
    rxbuflen = Serial1.available(); 
    if(rxbuflen > 0){
      *RxPacketPtr++ = Serial1.read();
      if(strlen(RxPacketBuf) < 5)
        continue;
      findptr1 = strstr(RxPacketBuf, "+IPD");
      if(findptr1 != NULL){
        findptr2 = strstr(findptr1, ":");
        if(findptr2 != NULL){
          Serial.println("colon(:) found");
          if(findptr2 > (findptr1+4)){
            memset(datalen, 0, 4);
            strncpy(datalen, (findptr1+4), (findptr2 - (findptr1+4)));          // +IPD543:
            Serial.print("message length (before atoi): ");
            Serial.println(datalen);
            ipdlen = atoi(datalen);    
            while(ipdlen){
              *RxPacketPtr = Serial1.read();
              Serial.print(RxPacketPtr);
              RxPacketPtr++;
              ipdlen--;
            }                                 
            findptr1 = strstr(RxPacketBuf, findbuffer);
            if(findptr1 != NULL){
              memset(RxPacketBuf, 0, RxBufSize);
              RxPacketPtr = &RxPacketBuf[0];
              return(1);
            }
          }
        }
      }
    }
  // check timeout
    mailtimeout++;
    delay(1000);
    if(mailtimeout > EMAILTIMEOUT){
      Serial.println("");
      Serial.println("Time Out occur!");
      modemRequest("AT+CIPCLOSE", 1, 1, "CLOSE OK");
      modemRequest("AT+CIPSHUT", 1, 1, "SHUT OK");
      RxPacketPtr = &RxPacketBuf[0];
      memset(RxPacketBuf, 0, RxBufSize);
      Serial.println("TimeOut After Connect OK");
      return(0);
    }
  }
  return(0);
}



/*****************************************************************************
 * Function: cmd_SMS
 *
 * Precondition: None.
 *
 * Overview: Execute a command from a SMS message
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

int cmd_SMS(void){
  String cmdSMS, senderPh, query;
  char command[20];
  int res;
  char tempbuf[185];
  char temp;
  
  if(!n_cmd_queue.isEmpty()){
    memset(command, 0, 20);
    cmdSMS = n_cmd_queue.pop();                // pop a message from queue
    Serial.println("Command to process is: ");
    Serial.println(cmdSMS);
    cmdSMS.toCharArray(command, cmdSMS.length()+1);
    if(strstr(command, "status") != NULL){
      Serial.println("Status Command");
      memset(tempbuf, 0, 50);
      senderPh = n_phone_queue.pop();
      query = String("AT+CMGS=\"");
      query += String("+5");
      senderPh.setCharAt(0, '8');
      query += senderPh;
      query += String("\"\r");      
      Serial.println("Read Eeprom Site Name");
//      int value;
//      String tmpsite;
//      uint16_t l = 0x171;
//      for(int n=0; n < strlen(sitename); n++){
//        value = EEPROM.read(l);
//        tmpsite += (char)value;
//        l++;        
//      }  
//      query += tmpsite;
      query += sitename;
      query += String(buffer_status);   
      query.toCharArray(tempbuf, query.length()+1);
      *(tempbuf+strlen(tempbuf)) = 0x1A;
      res = modemRequest(tempbuf, 5, 1, "OK");
      #if defined __debug_sms__
      if(res){
        Serial.println("Message sent!");  
      }else{
        Serial.println("Problem while trying to send message!");  
      }      
      #endif
      return(1);
    }else{
      // Nothing to do, SMS already erased.
      Serial.println("command not found");  
    }
  }
  return(0);
}


/*****************************************************************************
 * Function: DisableSMSIndications
 *
 * Precondition: None.
 *
 * Overview: Remove SMS indications
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

void DisableSMSIndications(void){
  Serial.println("Disable SMS indications");
  modemRequest("AT+CNMI=2,0,0,0,0", 1, 1, "OK");  
}


/*****************************************************************************
 * Function: EnableSMSIndications
 *
 * Precondition: None.
 *
 * Overview: Remove SMS indications
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

void EnableSMSIndications(void){
  Serial.println("Enable SMS indications");
  modemRequest("AT+CNMI=2,1,0,0,0", 1, 1, "OK");  
}
