
/****************************************************************************** 
 _______  _______  _        _______  _______  _______ 
(  ____ \(  ____ \( (    /|(  ____ \(  ___  )(  ____ )
| (    \/| (    \/|  \  ( || (    \/| (   ) || (    )|
| (_____ | (__    |   \ | || (_____ | |   | || (____)|
(_____  )|  __)   | (\ \) |(_____  )| |   | ||     __)
      ) || (      | | \   |      ) || |   | || (\ (   
/\____) || (____/\| )  \  |/\____) || (___) || ) \ \__
\_______)(_______/|/    )_)\_______)(_______)|/   \__/
                                                      
 _        _______ _________          _______  _______  _       
( (    /|(  ____ \\__   __/|\     /|(  ___  )(  ____ )| \    /\
|  \  ( || (    \/   ) (   | )   ( || (   ) || (    )||  \  / /
|   \ | || (__       | |   | | _ | || |   | || (____)||  (_/ / 
| (\ \) ||  __)      | |   | |( )| || |   | ||     __)|   _ (  
| | \   || (         | |   | || || || |   | || (\ (   |  ( \ \ 
| )  \  || (____/\   | |   | () () || (___) || ) \ \__|  /  \ \
|/    )_)(_______/   )_(   (_______)(_______)|/   \__/|_/    \/


// Network of Sensors supporting code
******************************************************************************/

// includes
#include "system.h"
#include <XBee.h>
#include <Sensors.h>


XBee xbee = XBee();
XBeeResponse response = XBeeResponse();
// create reusable response objects for responses we expect to handle 
ZBRxResponse rx = ZBRxResponse();
ModemStatusResponse msr = ModemStatusResponse();

ZBRxIoSampleResponse ioSample = ZBRxIoSampleResponse();

XBeeAddress64 device_addr = XBeeAddress64();

String ioSampleAdd, rxResponseAdd, Str2Cmp;
float tempC, light;
float mVanalog;

// Set DIO4 to Digital HIGH
uint8_t d4Cmd[] = { 'D', '4' };
uint8_t d4ValueHigh[] = { 0x5 };
uint8_t d4ValueLow[] = { 0x4 };

// Set DO8 to Digital HIGH
uint8_t d8Cmd[] = { 'D', '8' };
uint8_t d8ValueHigh[] = { 0x5 };
uint8_t d8ValueLow[] = { 0x4 };

// Set DIO9 to Digital HIGH
uint8_t d9Cmd[] = { 'D', '9' };
uint8_t d9ValueHigh[] = { 0x5 };
uint8_t d9ValueLow[] = { 0x4 };

char buffer_iot[IOT];
char* buf_ptr = &buffer_iot[0];
char buffer_status[160];                    // This size is the maximum allowed by SMS text message.
char* stat_ptr = &buffer_status[0];
uint8_t clear_buffer = 1;                  //  Value = 1 will clear the buffer when alarm is updated.      

// Broadcast Address
uint16_t unknowntAddr = 0xFFFE;
// Set DIO4 to Digital HIGH
uint8_t discoverCmd[] = { 'N', 'D' };            // Node Discovery Command (recommended only if the network is not large)
// Network Discovery Command
RemoteAtCommandRequest discoveryAtRequest = RemoteAtCommandRequest(unknowntAddr, discoverCmd);

uint8_t temp_type;
uint16_t temp_parent;
int next = 0;

// SH + SL of your remote radio
XBeeAddress64 remoteAddress = XBeeAddress64(0x0013A200, 0x4030C173);
XBeeAddress64 Addr2Cmp = XBeeAddress64();
// Create a remote AT request 
RemoteAtCommandRequest RemAtRequest = RemoteAtCommandRequest(remoteAddress, d4Cmd, d4ValueLow, sizeof(d4ValueLow));

// Create a Remote AT response object
RemoteAtCommandResponse remoteAtResponse = RemoteAtCommandResponse();

// Our Sensor Class Instances

UserSensorsList  UsrDefSensors;
XBeeDeviceBase*  devices[MAX_NUM_XBEE_SENSORS];
int dev;

int new_cosm_update = 0;

/*****************************************************************************
 * Function: InitSensorNetwork()
 *
 * Precondition: None
 *
 * Overview: Initialize the XBee sensor network
 *           
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

void InitSensorNetwork(void){
  Serial.println();
  Serial.println("Initializing the XBee Network Interface");
  delay(5000);
  uint8_t ch[] = {0,0,0,0,0,0,0,0,0,0,0,0,0};
  uint8_t ch1[] = {0,0,TEMPERATURE,0,0,0,0,0,0,0,0,0,0};
  uint8_t ch2[] = {0,0, DIGITAL, 0,0,0,0,0,0,0,0,0,0};
  char* datastreams[] = {"","","","","","","","","","","","",""};
  char* datastreams2[] = {"","","D1","","","","","","","","","",""};
  char* alarms[] = {"","","","","","","","","","","","",""};
  char* alarms2[] = {"","","Door Opened:0","","","","","","","","","",""};
  // let's hardwire the list of known sensors
  UsrDefSensors.add2list(XBeeAddress64(XBeeWallRouter_msb, XBeeWallRouter_lsb), Router, 0xFFFE, ch, datastreams, alarms, "", "");
  UsrDefSensors.add2list(XBeeAddress64(XBeeSensor_msb, XBeeSensor_lsb), EndDevice, 0xFFFE, ch1, datastreams, alarms, "temp1.csv", "");          // XBee Analog Sensor ADC2 input
  UsrDefSensors.add2list(XBeeAddress64(XBeeEnd_msb, XBeeEnd_lsb), EndDevice, 0xFFFE, ch2, datastreams2, alarms2, "dig1.csv", "99999");
  Serial.println("Send a Broadcast Node Discovery AT command");
  xbee.send(discoveryAtRequest);
  for(int i=0; i < MAX_NUM_XBEE_SENSORS;  i++){
    devices[i] = NULL;
  }
  strcat(buffer_status, "System Up and Running\n\r");   
  String time = getSysDateTime();
  char t[20];
  time.toCharArray(t, time.length()+1);
  strcat(buffer_status, "@ ");
  strcat(buffer_status, t);
  strcat(buffer_status, "\n\r");
  MainLogUpdate(buffer_status);
}

void ForcesSampleFnc(uint32_t msb, uint32_t lsb){
  Serial.println("Enter ForcesSampleFnc Method");
  uint8_t forceSampleCmd[] = { 'I', 'S' };
  XBeeAddress64 SensorAddress = XBeeAddress64(msb, lsb);
  RemoteAtCommandRequest ForceSampleAtRequest = RemoteAtCommandRequest(SensorAddress, forceSampleCmd);
  xbee.send(ForceSampleAtRequest);
}

void TurnLedXBIB(int led, int value){
  RemAtRequest.clearCommandValue();
  switch(led){
    case 1: 
      RemAtRequest.setCommand(d4Cmd);
      if(value == HIGH){
        RemAtRequest.setCommandValue(d4ValueHigh);
        RemAtRequest.setCommandValueLength(sizeof(d4ValueHigh));
      }else{
        RemAtRequest.setCommandValue(d4ValueLow);
        RemAtRequest.setCommandValueLength(sizeof(d4ValueLow));
      }
    break;  
    case 2:
      RemAtRequest.setCommand(d8Cmd);
      if(value == HIGH){
        RemAtRequest.setCommandValue(d8ValueHigh);
        RemAtRequest.setCommandValueLength(sizeof(d8ValueHigh));
      }else{
        RemAtRequest.setCommandValue(d8ValueLow);
        RemAtRequest.setCommandValueLength(sizeof(d8ValueLow));
      }
    break;
    case 3:
      RemAtRequest.setCommand(d9Cmd);
      if(value == HIGH){
        RemAtRequest.setCommandValue(d9ValueHigh);
        RemAtRequest.setCommandValueLength(sizeof(d9ValueHigh));
      }else{
        RemAtRequest.setCommandValue(d9ValueLow);
        RemAtRequest.setCommandValueLength(sizeof(d9ValueLow));
      }
    break;
  }

  // send the command
  xbee.send(RemAtRequest);  
}



/*****************************************************************************
 * Function: check_if_registered(XBeeAddress64 Addr2Cmp)
 *
 * Precondition: None
 *
 * Overview: 
 *           
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

int check_if_registered(void){      // I can get this compile as a parameter ... XBeeAddress64 Addr2Cmp
  for(int i=0; i < next; i++){
    if(Addr2Cmp == devices[i]->addr)
      return(devices[i]->type);
  }
  return(-1);
}

/*****************************************************************************
 * Function: Sensor Network Status SensorNetworkStatus
 *
 * Precondition: None
 *
 * Overview: 
 *           
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
 
int SensorNetworkStatus(String t){
    char time[20];
    char tmpbuf[160];
    char* tmpbuf_ptr = &tmpbuf[0];    
    
    Serial.println("Enter SensorNetworkStatus Method");
    memset(time,0, 20);
    memset(tmpbuf,0, 20);
    for(int i=0; i < MAX_NUM_XBEE_SENSORS;  i++){
      if(devices[i] == NULL){
        return(0); 
      }
      t.toCharArray(time, t.length()+1);
      if(devices[i]->GetSensorStatus(&tmpbuf_ptr, time) == 1){
        if(clear_buffer){
          memset(buffer_status, 0, 160);
          clear_buffer = 0;
        }
        Serial.println();
        Serial.println("**********************************************");
        Serial.println("Sensor Network Status (Console):");
        if(strlen(tmpbuf) == 0){
          strcat(tmpbuf, "System OK\n\r");
          strcat(tmpbuf, "@ ");
          strcat(tmpbuf, time);
          strcat(tmpbuf, "\n\r");
        }else{
          if((strlen(buffer_status) + strlen(tmpbuf)) > 160){          // if more than 160 characters, we need to erase previous messages...give new message a priority.
            memset(buffer_status, 0, 160);
          } 
        }
        strcat(buffer_status, tmpbuf);
        Serial.println(buffer_status);
        Serial.println("**********************************************");
        Serial.println();
        MainLogUpdate(tmpbuf);                     // Update Main Log File
        #if defined __GPRS_compile__
        send_SMS_Status();                         // Send Alarm Condition to List of SMS
        #endif
      }
    }
    return(1);
 }


void printHex(int num, int precision) {
      char tmp[16];
      char format[128];

      sprintf(format, "0x%%.%dX", precision);

      sprintf(tmp, format, num);
      Serial.print(tmp);
}
/*****************************************************************************
 * Function: 
 *
 * Precondition: None
 *
 * Overview: 
 *           
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
 
 
void XBeeNetworkLoop(void){
   
    xbee.readPacket();
  
  if (xbee.getResponse().isAvailable()) {
    // got something
    
    if (xbee.getResponse().getApiId() == ZB_IO_NODE_IDENTIFIER_RESPONSE) {
      // got a zb rx packet
      Serial.println("ZB_IO_NODE_IDENTIFIER_RESPONSE");
      // now fill our zb rx class
      xbee.getResponse().getZBRxResponse(rx);      
      device_addr = rx.getRemoteAddress64 ();
        Addr2Cmp = device_addr;      
        dev = check_if_registered();
      if(dev >= 0){
        #if defined __debug_xbee__
        if(dev == Router){
          Serial.println("Router Re-Joined!");  
        }else if(dev == EndDevice){
          Serial.println("XBee Sensor Re-Joined!");  
        }
        #endif
        // do nothing...
      }else{
        // should register joined device  
        temp_type = rx.getData(rx.getDataOffset () + 3);      // Fucking Bug - NULL terminators don't count.
        temp_parent = rx.getData(rx.getDataOffset () + 1) << 8 | rx.getData(rx.getDataOffset () + 2);   
        devices[next] = new XBeeDeviceBase(device_addr, temp_parent, temp_type);
        Serial.print("Device type is: ");
        Serial.println(devices[next]->NodeType());
        devices[next]->CreateSensor(&UsrDefSensors);          // should rework on this
        next++;        
      }
      // just some debugging
      #if defined __debug_xbee__
      Serial.print("device 64 address is ");
      Serial.print(device_addr.getMsb(), HEX);
      Serial.println(device_addr.getLsb(), HEX);
      if (rx.getDataLength () > 0){
        Serial.print("getting length = ");
        Serial.println(rx.getDataOffset (), HEX);
      }
      Serial.println("RX DATA:");
      for(int i=rx.getDataOffset (); i<rx.getDataLength(); i++){
        Serial.print(rx.getData(i), HEX);
        Serial.print(" ");  
      }
      Serial.print("\n"); 
      #endif
    }else if (xbee.getResponse().getApiId() == ZB_IO_SAMPLE_RESPONSE){
      Serial.println("ZB_IO_SAMPLE_RESPONSE");
      xbee.getResponse().getZBRxIoSampleResponse(ioSample); 
      device_addr = ioSample.getRemoteAddress64();
      Serial.print("Sensor is in list position # ");
      Serial.println(UsrDefSensors.findinlist(device_addr), DEC);

      int j = 1;
      int i = 0;
      for(i=0; i < MAX_NUM_XBEE_SENSORS;  i++){
        if(devices[i] == NULL){
          break; 
        }
        if(device_addr == devices[i]->addr){
          String t = getSysDateTime();
          Serial.println("calling ShowSample method");
          devices[i]->ShowSample(ioSample, t);
          Serial.println("return from ShowSample method");
          memset(buffer_iot, 0, IOT);
          new_cosm_update = devices[i]->BuildInternetOfThingsPUT(&buf_ptr, API_KEY);
          #if defined __dbg_pachube_qry__ 
          Serial.println("Let's print Internet Of Things Update");
	  Serial.print(buffer_iot);
          #endif        
          j = 0;
          break;
        }
      }
      if(j){            // a sample arrive and the device is not added to the know list of devices, in that case... 
        Serial.println("Device not in list");
        temp_type = EndDevice;                          // I should rework on this
        temp_parent = 0xfffe;                            
        devices[next] = new XBeeDeviceBase(device_addr, temp_parent, temp_type);
        Serial.print("Device type is: ");
        Serial.println(devices[next]->NodeType());
        devices[next]->CreateSensor(&UsrDefSensors);          // should rework on this
        String t = getSysDateTime();
        Serial.println("calling ShowSample method");
        devices[next]->ShowSample(ioSample, t);
        memset(buffer_iot, 0, IOT);
        new_cosm_update = devices[next]->BuildInternetOfThingsPUT(&buf_ptr, API_KEY);       
        next++;                         
      }
      
      #if defined __debug_xbee__
      Serial.print("Received I/O Sample from: ");  
      Serial.print(device_addr.getMsb(), HEX);
      Serial.println(device_addr.getLsb(), HEX);
      
      // debugging purpose

      if (ioSample.containsAnalog()) {
        Serial.print("Sample contains analog data, mask: ");
        Serial.println(ioSample.getAnalogMask(), BIN);
      }

      if (ioSample.containsDigital()) {
        Serial.print("Sample contains digtal data, mask MSB: ");
        Serial.println(ioSample.getDigitalMaskMsb(), BIN);
        Serial.print("mask LSB: ");
        Serial.println(ioSample.getDigitalMaskLsb(), BIN);
      }      
      
      // read analog inputs
      for (int i = 0; i <= 4; i++) {
        if (ioSample.isAnalogEnabled(i)) {
          Serial.print("Analog (AI");
          Serial.print(i, DEC);
          Serial.print(") is ");
          Serial.println(ioSample.getAnalog(i), DEC);
        }
      }
      
      if(ioSample.isAnalogEnabled(7)){
          Serial.print("%V (AI");
          Serial.print(7, DEC);
          Serial.print(") is ");
          Serial.println(ioSample.getAnalog(7), DEC);
      }

      // check digital inputs
      for (int i = 0; i <= 12; i++) {
        if (ioSample.isDigitalEnabled(i)) {
          Serial.print("Digital (DI");
          Serial.print(i, DEC);
          Serial.print(") is ");
          Serial.println(ioSample.isDigitalOn(i), DEC);
        }
      }
      #endif      
    }else if (xbee.getResponse().getApiId() == REMOTE_AT_COMMAND_RESPONSE) {    // should be an AT command response
      Serial.println("REMOTE_AT_COMMAND_RESPONSE");
      xbee.getResponse().getRemoteAtCommandResponse(remoteAtResponse);
      char *rxframe = (char*)xbee.getResponse().getFrameData();
      Serial.println("Frame Data is: ");
      for(int j=0; j < xbee.getResponse().getFrameDataLength(); j++){
        Serial.print((unsigned char)*rxframe++, HEX);
        Serial.print(" ");
      }
      Serial.println("");
      if (remoteAtResponse.isOk()) {
        Serial.print("Command [");
        Serial.print(remoteAtResponse.getCommand()[0]);
        Serial.print(remoteAtResponse.getCommand()[1]);
        Serial.println("] was successful!");
        
        // if it's a response of Node Discovery Command then
        if(remoteAtResponse.getCommand()[0] == 'N' && remoteAtResponse.getCommand()[1] == 'D'){
          device_addr = remoteAtResponse.getRemoteAddress64();
          Addr2Cmp = device_addr;
          dev = check_if_registered();     
          if(dev >= 0){
            if(dev == Router){
              Serial.println("A Router Device already added with address: ");  
            }else if(dev == EndDevice){
              Serial.println("An End Device already added with address: ");  
            }
            Serial.print("0x");
            Serial.print(device_addr.getMsb(), HEX);
            Serial.print(" 0x");
            Serial.println(device_addr.getLsb(), HEX);            
          }else{
            uint8_t *AtCmdResponse;
            AtCmdResponse = remoteAtResponse.getValue ();
            AtCmdResponse += 10;                                                  // Point to first byte of NI String.
            while(*AtCmdResponse != 0x00)
              AtCmdResponse++;
            AtCmdResponse++;                                                      // Point to Parent 16 Bit Address first byte.
            temp_parent = *AtCmdResponse++;
            temp_parent <<= 8;
            temp_parent |= *AtCmdResponse++;
            temp_type = *AtCmdResponse;
            
            devices[next] = new XBeeDeviceBase(device_addr, temp_parent, temp_type);
            Serial.print("New ");
            Serial.print(devices[next]->NodeType());     
            Serial.print(" added to the Network list with address: 0x");  
            Serial.print(devices[next]->addr.getMsb(), HEX);
            Serial.print(" 0x");
            Serial.println(devices[next]->addr.getLsb(), HEX);
            devices[next]->CreateSensor(&UsrDefSensors);
            next++;
          }
        }

      } else if(remoteAtResponse.getCommand()[0] == 'I' && remoteAtResponse.getCommand()[1] == 'S'){
        
        
      } else {
        Serial.print("Command returned error code: ");
        Serial.println(remoteAtResponse.getStatus(), HEX);
        Serial.print("Command was: ");
        Serial.print(remoteAtResponse.getCommand()[0]);
        Serial.print(remoteAtResponse.getCommand()[1]);
        Serial.println("");        
      }
    }
  }else if (xbee.getResponse().isError()) {
    Serial.print("Error reading packet.  Error code: ");  
    Serial.println(xbee.getResponse().getErrorCode(), DEC);
    for (int i = 0; i < rx.getDataLength(); i++) { 
      Serial.print(rx.getData(i), BYTE); 
    } 
  }
}
 


