/****************************************************************************** 
 _______  _______  _______  _______  _______          
(       )(  ____ \(       )(  ___  )(  ____ )|\     /|
| () () || (    \/| () () || (   ) || (    )|( \   / )
| || || || (__    | || || || |   | || (____)| \ (_) / 
| |(_)| ||  __)   | |(_)| || |   | ||     __)  \   /  
| |   | || (      | |   | || |   | || (\ (      ) (   
| )   ( || (____/\| )   ( || (___) || ) \ \__   | |   
|/     \|(_______/|/     \|(_______)|/   \__/   \_/   
                                                      
          _______  _        ______   _        _______  _______ 
|\     /|(  ___  )( (    /|(  __  \ ( \      (  ____ \(  ____ )
| )   ( || (   ) ||  \  ( || (  \  )| (      | (    \/| (    )|
| (___) || (___) ||   \ | || |   ) || |      | (__    | (____)|
|  ___  ||  ___  || (\ \) || |   | || |      |  __)   |     __)
| (   ) || (   ) || | \   || |   ) || |      | (      | (\ (   
| )   ( || )   ( || )  \  || (__/  )| (____/\| (____/\| ) \ \__
|/     \||/     \||/    )_)(______/ (_______/(_______/|/   \__/
                                                               
******************************************************************************/
                                                              

// Memory Managment Functions

#include "system.h"
#include <MCP794xxRTC.h>    // a basic MCP794xx library
#include <EEPROM.h>


MCP794xxRTC RTC = MCP794xxRTC(2); // create an instance of device MCP79402


uint8_t entry_num=0;                      // this is the number of entries of phone numbers we have in memory
uint8_t temp_addr;                      // this is the last available address to save the next entry, one byte will give us more or less than 20 entries depending on ph number length
uint8_t addr_last;                       // this is the eeprom address variable we will use to save the phone numbers...
uint8_t addr_last_msb;                       // this is the eeprom address variable we will use to save the phone numbers...
uint8_t addr_last_lsb;                       // this is the eeprom address variable we will use to save the phone numbers...
//String  query_value;                     // phone number value of query entry 
uint16_t temp_addr_two; 

void SaveEntryMem(String &tempPh){
  char tph[15];
  
  RTC.read_byte(eeprom_phone_entry_last, addr_last);           // find the last available position in eeprom to save a new entry
  tempPh.toCharArray(tph, tempPh.length()+1);                  // convert to char array...            
  for(int j=0; j<tempPh.length(); j++){
    EEPROM.write(addr_last, tph[j]);                           // save in eeprom
    addr_last++;                                               
  }                                                            // next position should be left blank
  //EEPROM.write(addr_last, 0x00);                             // signal start/end of register    
  addr_last++;                                                 // plus the signal end (leave a blank byte in between entries)
  RTC.set_address(eeprom_phone_entry_last);                    // save the address of last position (free or available to continue with next entry)
  RTC.write_byte(addr_last);                                   // save the address
  RTC.read_byte(eeprom_phone_entry_num, entry_num);            // get the number of entries old value
  entry_num++;                                                 // a new entry was added so add 1 to the old value
  RTC.set_address(eeprom_phone_entry_num);                     // set the SRAM address of the number of entries
  RTC.write_byte(entry_num);                                   // finally save the value
  
  Serial.println("Reading current values from RTC memory");
  RTC.read_byte(eeprom_phone_entry_num, entry_num);            // get the number of entries old value
  RTC.read_byte(eeprom_phone_entry_last, addr_last);            // get the number of entries old value
  #if defined __sram_wr_debug__
  Serial.print("eeprom_phone_entry_last value is= ");          // some debugging...
  Serial.println(addr_last, HEX);
  Serial.print("eeprom_phone_entry_num value is= ");
  Serial.println(entry_num, HEX);
  #endif
}


void SaveEmailMem(String &tempEmail){
  char tmail[40];
  uint16_t address;
  uint8_t tadd;
  
  RTC.read_byte(eeprom_mail_entry_last_msb, addr_last_msb);           // find the last available position in eeprom to save a new entry
  RTC.read_byte(eeprom_mail_entry_last_lsb, addr_last_lsb);           // find the last available position in eeprom to save a new entry  
  address = ((addr_last_msb << 8) | addr_last_lsb);
  
  Serial.println();
  Serial.println("address is");
  Serial.println(address, DEC);
  Serial.println();
  
  // I should include here a comparison if address is >= 600 (see queryEmails) and abort in such a case.
  
  tempEmail.toCharArray(tmail, tempEmail.length()+1);         // convert to char array...            
  for(int j=0; j<tempEmail.length(); j++){
    EEPROM.write(address, tmail[j]);                           // save in eeprom
    Serial.print(tmail[j]);
    address++;                                               
  }                                                            // next position should be left blank
  //EEPROM.write(addr_last, 0x00);                             // signal start/end of register    
  address++;                                                   // plus the signal end (leave a blank byte in between entries)
  RTC.set_address(eeprom_mail_entry_last_msb);                // save the address of last position (free or available to continue with next entry)
  tadd = (uint8_t)((address & 0xFF00) >> 8);
  Serial.println();
  Serial.println("tadd msb nibble is");
  Serial.println(tadd, HEX);
  Serial.println();
  RTC.write_byte(tadd);                                        // save the address

  RTC.set_address(eeprom_mail_entry_last_lsb);                // save the address of last position (free or available to continue with next entry)
  tadd = (uint8_t)(address & 0x00FF);
  Serial.println();
  Serial.println("tadd lsb nibble is");
  Serial.println(tadd, HEX);
  Serial.println();
  RTC.write_byte(tadd);  
  
  RTC.read_byte(eeprom_mail_entry_num, entry_num);            // get the number of entries old value
  entry_num++;                                                 // a new entry was added so add 1 to the old value
  RTC.set_address(eeprom_mail_entry_num);                     // set the SRAM address of the number of entries
  RTC.write_byte(entry_num);                                   // finally save the value
  
  #if defined __sram_wr_debug__
  Serial.println();
  Serial.print("eeprom_mail_entry_last value is= ");          // some debugging...
  Serial.println(address, HEX);
  Serial.print("eeprom_mail_entry_num value is= ");
  Serial.println(entry_num, HEX);
  Serial.println("address is");
  Serial.println(address, DEC);
  Serial.println();
  #endif
}


void InitProgState(void){

  RTC.set_address(0);
  for(int i=0; i<64; i++){
    RTC.write_byte(0x00); 
  }
  for(int i=0; i<512; i++){
    EEPROM.write(i, 0x00); 
  }
  
  // set some known init MUST state
  RTC.set_address(eeprom_phone_entry_last);
  RTC.write_byte(0x01);                                  // First position should contain a zero! So last address value is 1
  RTC.set_address(eeprom_mail_entry_last_msb);
  RTC.write_byte(0x01);                                  // 256 is the initial address.
  // get some state and working variables
  RTC.read_byte(eeprom_phone_entry_num, entry_num);       
  RTC.read_byte(eeprom_phone_entry_last, addr_last);      
  #if defined __sram_wr_debug__
  Serial.print("eeprom_phone_entry_last value is= ");
  Serial.println(addr_last, HEX);
  Serial.print("eeprom_phone_entry_num value is= ");
  Serial.println(entry_num, HEX);
  #endif
//  EraseSMS();
}


/*****************************************************************************
 * Function: getNumberOfEntries()
 *
 * Precondition: None.
 *
 * Overview: Get the actual pointer to current availabel space for new entry
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
 
 uint8_t getNumberOfEntries(void){
   
  uint8_t i; 
  RTC.read_byte(eeprom_phone_entry_num, i);                  // read current entry number
  return(i);
 }
 
 /*****************************************************************************
 * Function: getNumberOfEntriesMail()
 *
 * Precondition: None.
 *
 * Overview: Get the actual pointer to current availabel space for new entry
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
 
 uint8_t getNumberOfEntriesMail(void){
   
  uint8_t i; 
  RTC.read_byte(eeprom_mail_entry_num, i);                  // read current entry number
  return(i);
 }
 
 
/*****************************************************************************
 * Function: QueryEntry
 *
 * Precondition: None.
 *
 * Overview: Query a phone number value. 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

String QueryEmailEntry(int query_entry){
  int entry_count, j;
  char temp;
  String query_value;

  RTC.read_byte(eeprom_mail_entry_num, entry_num);                  // read current entry number
  #if defined __sram_wr_debug__
  Serial.print("Query Number requested: ");
  Serial.println(query_entry, DEC);
  #endif        
  if(entry_num < query_entry){                                       // not a valid query
    #if defined __sram_wr_debug__
    Serial.print("No such an entry!");
    #endif
  }else{
    #if defined __sram_wr_debug__
    Serial.println("searching record!");                               // record is in memory, we need to search it!
    #endif
    temp_addr_two = 255;                                                   // temporal address
    entry_count = 0;                                                 // temporal count of read registers
    j = 1;
    while(j){
      temp = (char)EEPROM.read(temp_addr_two);                          // get value from eeprom
      temp_addr_two++;                                                  // advance address
      if(temp == 0x00){                                             // if it's the start of phone number entry
        if(query_entry == entry_count){                              // and query correspond to this query
          temp = (char)EEPROM.read(temp_addr_two);                      // read eeprom
          Serial.print(temp);
          temp_addr_two++;
          while(temp != 0x00){                                       // extract until start of next query appears
            query_value += temp;
            temp = (char)EEPROM.read(temp_addr_two);                     // read eeprom     
            Serial.print(temp);
            temp_addr_two++;                                             // move to the address
          }                                                          // append char to query string
          #if defined __sram_wr_debug__
          Serial.println();
          Serial.print("query value is: ");
          Serial.println(query_value);
          #endif
          j = 0;             // leave the loop
          return(query_value);
        }else{
          entry_count++;                                            // if not, increment the count until a match!
        }
      }// loop until 
      if(temp_addr >= 600){            // 14 emails of 40 chars 
        j = 0;              // this should be safe...
        #if defined __sram_wr_debug__
        Serial.println("empty directory");
        #endif
      }
    }
  }
  return(query_value);
}

/*****************************************************************************
 * Function: QueryEntry
 *
 * Precondition: None.
 *
 * Overview: Query a phone number value. 
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/

void QueryEntry(char qvalue[], int query_entry){
  int entry_count, j;
  char temp;
  char *query_value = qvalue;
//  query_value = String();

  RTC.read_byte(eeprom_phone_entry_num, entry_num);                  // read current entry number
  #if defined __sram_wr_debug__
  Serial.print("Query Number requested: ");
  Serial.println(query_entry, DEC);
  #endif        
  if(entry_num < query_entry){                                       // not a valid query
    #if defined __sram_wr_debug__
    Serial.print("No such an entry!");
    #endif
  }else{
    #if defined __sram_wr_debug__
    Serial.println("searching record!");                               // record is in memory, we need to search it!
    #endif
    temp_addr = 0;                                                   // temporal address
    entry_count = 0;                                                 // temporal count of read registers
    j = 1;
    while(j){
      temp = (char)EEPROM.read(temp_addr);                          // get value from eeprom
      temp_addr++;                                                    // advance address
      if(temp == 0x00){                                             // if it's the start of phone number entry
        if(query_entry == entry_count){                              // and query correspond to this query
          temp = (char)EEPROM.read(temp_addr);                      // read eeprom
          temp_addr++;
          while(temp != 0x00){                                       // extract until start of next query appears
            *query_value = temp;
            query_value++;
            temp = (char)EEPROM.read(temp_addr);                     // read eeprom     
            temp_addr++;                                             // move to the address
          }                                                          // append char to query string
          #if defined __sram_wr_debug__
          Serial.print("query value is: ");
          Serial.println(qvalue);
          #endif
          j = 0;                                                    // leave the loop
        }else{
          entry_count++;                                            // if not, increment the count until a match!
        }
      }// loop until 
      if(temp_addr >= 255){
        j = 0;              // this should be safe...
        #if defined __sram_wr_debug__
        Serial.println("empty directory");
        #endif
      }
    }
  }
}

/*****************************************************************************
 * Function: is_in_Dir
 *
 * Precondition: None.
 *
 * Overview: look for a given phone number in memory
 *
 * Input: phone number String Object
 *
 * Output: if phone number is found return the corresponding entry number or -1 if not found.
 *
 *****************************************************************************/
 
int is_in_Dir(String phone){
  char ph2search[15];
  char mementry[15];
  
  int i = 0;
  
  phone.toCharArray(ph2search, phone.length()+1); 
  RTC.read_byte(eeprom_phone_entry_num, entry_num);                // get the number of entries in memory
  if(entry_num == 0)
    return(-1);
  for(i=0; i<(int)entry_num; i++){
    QueryEntry(mementry, i);
    if(strstr(mementry, ph2search) != NULL){
      break;  
    }
  }
  return(i);
}

