

#include "mbed.h"
AnalogIn ain(PTB0); 
DigitalIn InFwd(PTB1), InRev(PTB2); // InFwd, InRev  initially Hi.
DigitalOut ledBlu(LED_BLUE), ledRed(LED_RED), ledGrn(LED_GREEN), OutFwd(PTE20), OutRev(PTE21);
//Serial pc(USBTX, USBRX); // tx, rx
#define TRUE -1
#define FALSE 0
#define MAX_IN_WAIT_COUNT 30 // for 30 ms = 30 * 0.001
#define MAX_MOT_CURRENT_WAIT 300 // for 3 Seconds = 300 * 0.01
#define MAXSAMPLES 30
#define NORMAL_CURRENT_THRESHOLD 0.9
#define HIGH_CURRENT_THRESHOLD  0.33
#define DEBUG


float fValue, fV, fX[MAXSAMPLES], fY[MAXSAMPLES], fDeltaT = 0.0003;
enum Motor
{   OFF,
    FWD,
    FWD_WITH_CURRENT,
    REV,
    REV_WITH_CURRENT
};

enum MotorCurrentType
{   NO_CURRENT,
    NORMAL_CURRENT,
    OVER_CURRENT
};
// sample Analog Input Pin and initialize Y[] array for low pass filter
void fGetAnalogIn(float *fX, float *fY)
{   unsigned i;
    for(i = 0; i < MAXSAMPLES; i++)
    {   wait(fDeltaT);
        fX[i] = ain;
        fY[i] = 1.0; // initialize Y values
    }
}

// implement simple low pass filter of form y[n] = y[n-1] + k(x[n-1] - y[n-1])
// where k = 1/T.  T is the time constant like RC in an analog filter
// return the minimum value in the data
#ifdef DEBUG
unsigned int uiDebugLoopCntr;
#endif  
float fLowPassFilter(float *fX, float *fY, float k)
{   unsigned int i, uiV;
    float fV = 1.0;
    for(i = 1; i < MAXSAMPLES -1; i++)
    {   fY[i] = fY[i-1]+k*(fX[i-1]-fY[i-1]);
        uiV = fY[i] * 100; fY[i] = (float) uiV/100; // only 2 digits after decimal
#ifdef DEBUG
        if(uiDebugLoopCntr == 0)
        {   printf( "i , x[i], y[i] %d %f %f\r\n",i,fX[i], fY[i]);
        }
#endif
        // now find minimum
        fV = 1.0;
        for(i = 1; i < MAXSAMPLES -1; i++)  
        { if( fY[i] < fV )
            fV = fY[i];
        }   
     }
     return(fV);
}

// See if there is motor current
// Return enum MotorCurrent 
MotorCurrentType eMotorCurrentCheck(float *fX, float *fY, float k)
{   float fMotCur = 1.0;
    MotorCurrentType MotorCurrent = NO_CURRENT;
    fGetAnalogIn(fX, fY);
    fMotCur = fLowPassFilter(fX, fY, k);
    if(fMotCur < NORMAL_CURRENT_THRESHOLD )
        MotorCurrent = NORMAL_CURRENT;
    if(fMotCur < HIGH_CURRENT_THRESHOLD )
        MotorCurrent = OVER_CURRENT;        
    return(MotorCurrent);
}
// basic input debounce routine
// must remain low for iMaxCount milliseconds
// Returns with iCntr == 0 when switch is closed OR iCntr == MaxCount
void debounce(unsigned int &iCntr, DigitalIn &TestParam, unsigned int iMaxCount)
{   iCntr = iMaxCount;
    do
    {   iCntr --;
        wait(0.001);
        if(TestParam == 0 )
           iCntr --;
        else 
            iCntr = iMaxCount;
     }while(iCntr > 0 && iCntr < iMaxCount);
}

int main()   
{   unsigned int iInCntrWait = MAX_IN_WAIT_COUNT; // to debounce inputs forward switch, reverse switch
    unsigned int iMotCurrentCntr = MAX_MOT_CURRENT_WAIT ;
    Motor power = OFF;// to control state machine
    float k = (1.0/0.0015)* fDeltaT; // 1.5 ms time constant for low pass filter
    MotorCurrentType eMotCurrent;
    
    OutFwd = 0; OutRev = 0; ledRed = 1; ledBlu = 1; ledGrn =0;
    //printf("Program Start!\n");
    wait(1.25);
    while(1) 
    {   // normal condition of inputs is High or == 1. Pulled low by switch
        switch(power)
        {
        case OFF:
            
            if(InFwd == 0 ) 
            {   // Forward button pushed
                debounce(iInCntrWait , InFwd, MAX_IN_WAIT_COUNT);
                if(iInCntrWait == 0)
                {   // debounce accomplished
                    ledGrn = 1; ledRed = 1; // turn off other LEDs
                    ledBlu = 0;
                    OutFwd = 1; // start motor going forward
                    power = FWD;
                    //printf("PTB0 enabled!\r\n");
                    wait(0.5); // give time for motor to get up to speed
                }
            }
            else
            {   if(InRev == 0 ) 
                {   // Reverse button pushed
                    debounce(iInCntrWait, InRev, MAX_IN_WAIT_COUNT);
                    if(iInCntrWait == 0)
                    {   // debounce accomplished
                        ledGrn = 1; ledBlu = 1; // turn off other LEDs
                        ledRed = 0;
                        OutRev = 1; // start motor going in reverse
                        power = REV;
                        //printf("PTB1 enabled!\r\n");
                        wait(0.5); // give time for motor to get up to speed
                    }
                } 
            } // endif(InFwd == 0 )
                      
        case FWD: 
            // Motor just started forward. Lets wait n seconds or until current is flowing in Motor circuit
            iMotCurrentCntr = MAX_MOT_CURRENT_WAIT;
            do
            {   wait(0.01);
                iMotCurrentCntr --;
                eMotCurrent = eMotorCurrentCheck(fX, fY, k);
            }while(iMotCurrentCntr > 0 || eMotCurrent == NO_CURRENT); // test if motor current is flowing
            if(iMotCurrentCntr == 0)
            {   // Motor hasn't started
                OutFwd = 0;
                ledRed = 1; ledBlu = 1; ledGrn =0;
                iInCntrWait  = MAX_IN_WAIT_COUNT;
                power = OFF; // back to start
            }
            else
                power = FWD_WITH_CURRENT;
                
             break;
             
          case FWD_WITH_CURRENT: 
            // Motor is running and current is flowing
             ledRed = 1; ledBlu = 0; ledGrn =1; // blue led ON
            iMotCurrentCntr = MAX_MOT_CURRENT_WAIT; // time to wait for current check
            if(InFwd == 0)
            {   // Forward switch may have pushed again
                debounce(iInCntrWait , InFwd, MAX_IN_WAIT_COUNT);
                if(iInCntrWait  == 0)
                {   // debounce accomplished so stop the motor
                    OutFwd = 0;
                    ledRed = 1; ledBlu = 1; ledGrn =0;
                    iInCntrWait = MAX_IN_WAIT_COUNT;
                    power = OFF; // back to start
                }
            }
            else
            {  // loops through here as long as motor is running
                do
                {   wait(0.01);
                    iMotCurrentCntr --;
                    eMotCurrent = eMotorCurrentCheck(fX, fY, k);
                }while(iMotCurrentCntr > 0 || eMotCurrent == NO_CURRENT); // test if motor current is flowing
                if(iMotCurrentCntr == 0)
                {   // hasn't been motor current signal for n seconds so back to stop
                    OutFwd = 0;
                    ledRed = 1; ledBlu = 1; ledGrn =0;
                    iInCntrWait = MAX_IN_WAIT_COUNT;
                    power = OFF; // back to start
                }
                else
                {   if(eMotCurrent == OVER_CURRENT)
                    {   int i, HighCur = TRUE; // check it 3 more times
                        for(i = 0 ; i < 3; i++)
                        { eMotCurrent = eMotorCurrentCheck(fX, fY, k);
                            if( eMotCurrent != OVER_CURRENT)
                            {   HighCur = FALSE;
                            }
                            if(HighCur)
                            {   // shut it down
                                OutFwd = 0;
                                ledRed = 1; ledBlu = 1; ledGrn =0;
                                iInCntrWait = MAX_IN_WAIT_COUNT;
                                power = OFF; // back to start                            
                            }
                        }  
                    }// endif(eMotCurrent == HIGH_CURRENT)
                }// endif(iMotCurrentCntr == 0)
            }// endif(InFwd == 0)
            break; 
             
        case REV:
            // Motor just started in reverse. Lets wait n seconds or until current is flowing in Motor circuit
            iMotCurrentCntr = MAX_MOT_CURRENT_WAIT;
            do
            {   wait(0.01);
                iMotCurrentCntr --;
                eMotCurrent = eMotorCurrentCheck(fX, fY, k);
            }while(iMotCurrentCntr > 0 || eMotCurrent == NO_CURRENT); // test if motor current is flowing
            if(iMotCurrentCntr == 0)
            {   // Motor hasn't started
                OutRev = 0;
                ledRed = 1; ledBlu = 1; ledGrn =0;
                iInCntrWait  = MAX_IN_WAIT_COUNT;
                power = OFF; // back to start
            }
            else
                power = REV_WITH_CURRENT;
                
            break; 
           
         case REV_WITH_CURRENT:
            // Motor is running and current is flowing
            ledRed = 0; ledBlu = 1; ledGrn =1; // red led ON
            iMotCurrentCntr = MAX_MOT_CURRENT_WAIT; // time to wait for current check
            if(InRev == 0)
            {   // Reverse switch may have pushed again
                debounce(iInCntrWait , InFwd, MAX_IN_WAIT_COUNT);
                if(iInCntrWait  == 0)
                {   // debounce accomplished so stop the motor
                    OutRev = 0;
                    ledRed = 1; ledBlu = 1; ledGrn =0;
                    iInCntrWait = MAX_IN_WAIT_COUNT;
                    power = OFF; // back to start
                }
            }
            else
            {  // loops through here as long as motor is running
                do
                {   wait(0.01);
                    iMotCurrentCntr --;
                    eMotCurrent = eMotorCurrentCheck(fX, fY, k);
                }while(iMotCurrentCntr > 0 || eMotCurrent == NO_CURRENT); // test if motor current is flowing
                if(iMotCurrentCntr == 0)
                {   // hasn't been motor current signal for n seconds so back to stop
                    OutRev = 0;
                    ledRed = 1; ledBlu = 1; ledGrn =0;
                    iInCntrWait = MAX_IN_WAIT_COUNT;
                    power = OFF; // back to start
                }
                else
                {   if(eMotCurrent == OVER_CURRENT)
                    {   int i, HighCur = TRUE; // check it 3 more times
                        for(i = 0 ; i < 3; i++)
                        {   eMotCurrent = eMotorCurrentCheck(fX, fY, k);
                            if( eMotCurrent != OVER_CURRENT)
                            {   HighCur = FALSE;
                            }
                            if(HighCur)
                            {   // shut it down
                                OutRev = 0;
                                ledRed = 1; ledBlu = 1; ledGrn =0;
                                iInCntrWait = MAX_IN_WAIT_COUNT;
                                power = OFF; // back to start                            
                            }
                        }  
                    }// endif(eMotCurrent == HIGH_CURRENT)
                }// endif(iMotCurrentCntr == 0)
            }// endif(InFwd == 0)
           break; 
                   
        default:
             break;
        } // endswitch(power)
        wait(0.1);
    } // endwhile(1)
}