/*
 * MCP794xxRTC.h - library for MCP794xx RTC/SRAM family of devices
  
  Copyright (c) Manuel Iglesias 2012
  This library is intended to be use with Arduino Time.h library functions and with
  Wire.h library (need to modify BUFFER_LENGTH to allow more than 64 bytes of reception and transmision buffer)
  bufer of the Inter Intergrated Library also need to allow at least 65 bytes to send.

  The library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  14 March 2012 - Initial release
 */



#ifndef MCP794xxRTC_h
#define MCP794xxRTC_h

#include <Time.h>

// library interface description
class MCP794xxRTC{
	int _part;
	int _address;
	int _ee_address;
  // user-accessible "public" interface
  public:
    MCP794xxRTC(int part);
    static time_t get();
	static void set(time_t t);
	static void read(tmElements_t &tm);
	static void write(tmElements_t &tm);
	void read_byte(uint8_t address, uint8_t &byte);
	void read_sram(uint8_t *byte);
	int  write_sram(uint8_t address, uint8_t *byte, uint8_t len);
	void write_byte(uint8_t &byte);
	void write_byte(int byte);
	void set_address(uint8_t address);
	void get_unique_id(uint8_t* uid);
	void get_eeprom_sequential(uint8_t* buf, uint8_t len);
  private:
	static uint8_t dec2bcd(uint8_t num);
    static uint8_t bcd2dec(uint8_t num);
};

#endif
 

