/*
 * Sensors.h - library for XBee Sensor definitions and managment
  
  Copyright (c) Manuel Iglesias 2012
  This library is intended to be uses with Arduino Time.h library functions

  The library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  21 March 2012 - Initial release
 */

#define DEBUG_SENSORS	1

#if ARDUINO >= 100
#include <Arduino.h> 
#else
#include <WProgram.h> 
#endif

#include "Sensors.h"
#include <XBee.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h> 
#include <string.h>
#include "TinyWebServer.h"


char *ftoa(char *a, double f)
{
  char *ret = a;
  long heiltal = (long)f;
  itoa(heiltal, a, 10);
  while (*a != '\0') a++;
  *a++ = '.';
  long desimal = abs((long)((f - heiltal) * 10));
  if(desimal < 1)
	*a++ = '0';
  else
	itoa(desimal, a, 10);
  return ret;
}

UserSensorsList::UserSensorsList(){
	_count = 0;		  
	for(uint8_t cnt=0; cnt<MAX_NUM_XBEE_SENSORS; cnt++){
		XBeeSensors[cnt] = NULL;
	}
}

void UserSensorsList::SetRootHandler(SdFile* root){
	this->root = root;
}

void UserSensorsList::add2list(XBeeAddress64 ADDR, uint8_t TYPE, uint16_t PARENT, uint8_t ch2add[], char* ds2add[], char* alarms2add[], char *fname, char *channel){
	
	Serial.println("Adding new sensor to the list");
	XBeeSensors[_count] = new UserDefXBeeSensors; 
	XBeeSensors[_count]->addr = ADDR;
	XBeeSensors[_count]->type = TYPE;
	XBeeSensors[_count]->parent = PARENT;
	XBeeSensors[_count]->root = root;
	if(*fname != '\0'){
		Serial.print("size of fname is: ");
		Serial.println(strlen(fname));
		void* r = malloc(strlen(fname)+1);					
		if(!r){
			XBeeSensors[_count]->filename = NULL;
		}else{
			XBeeSensors[_count]->filename = (char*)r;		// cast to char
			strcpy(XBeeSensors[_count]->filename, fname);	// copy fname parameter including null terminator
			//*(XBeeSensors[_count]->filename+1) = 0;			// NULL terminator
			Serial.print("filename is: ");
			Serial.println(XBeeSensors[_count]->filename);
		}
		char* rlog;
		rlog = (char*)malloc_check(LOG);
		if(!rlog){
			Serial.println("Could not allocate log memory");
			XBeeSensors[_count]->log = NULL;
		}else{
			Serial.println("log memory allocation OK");
			XBeeSensors[_count]->log = rlog;			// cast to char
			memset(XBeeSensors[_count]->log, 0, LOG);		
		}
	}else{
		Serial.println("Sensor fname is empty");
		XBeeSensors[_count]->filename = NULL;
		XBeeSensors[_count]->log = NULL;
	}
	if(ch2add != NULL){
		char* value;
		Serial.println("channel definition input not null");
		for(uint8_t j=0; j<MAXSIZE; j++){
			XBeeSensors[_count]->chdefs[j] = ch2add[j];
			if(ch2add[j] != 0){
				value = (char*)malloc_check(strlen(ds2add[j]) + 1);
				if(!value)
					continue;
				else{
					XBeeSensors[_count]->dsdefs[j] = value;
					strcpy(XBeeSensors[_count]->dsdefs[j], ds2add[j]);
					*((XBeeSensors[_count]->dsdefs[j])+strlen(ds2add[j])+1) = '\0';
				}
			}else{
				XBeeSensors[_count]->dsdefs[j] = NULL;
			}
		}
		value = (char*)malloc_check(strlen(channel) + 1);
		if(!value)
			XBeeSensors[_count]->channel = NULL;
		else{		
			XBeeSensors[_count]->channel = value;
			strcpy(XBeeSensors[_count]->channel, channel);
			*((XBeeSensors[_count]->channel)+strlen(channel)+1) = '\0';
		}
		char* csv;
		csv = (char*)malloc_check(CSV);	
		XBeeSensors[_count]->csv = csv;
		if(!csv){
			Serial.println("Could not allocate csv memory");
			XBeeSensors[_count]->csv = NULL;
		}else{		
			XBeeSensors[_count]->csv = csv;
			memset(XBeeSensors[_count]->csv, 0, CSV);	
		}
		char* val;
		for(uint8_t j=0; j<MAXSIZE; j++){
			if(strlen(alarms2add[j]) > 0){
				val = (char*)malloc_check(strlen(alarms2add[j]) + 1);
				if(!val)
					continue;
				else{
					XBeeSensors[_count]->alarmdefs[j] = val;
					strcpy(XBeeSensors[_count]->alarmdefs[j], alarms2add[j]);
					*((XBeeSensors[_count]->alarmdefs[j])+strlen(alarms2add[j])+1) = '\0';
				}
			}else{						
				XBeeSensors[_count]->alarmdefs[j] = NULL;
			}
		}		
		
	}else{
		Serial.println("digital input null");
		for(uint8_t j=0; j<MAXSIZE; j++){
			XBeeSensors[_count]->chdefs[j] = 255;
			XBeeSensors[_count]->dsdefs[j] = NULL;
		}
		XBeeSensors[_count]->csv = NULL;
	}
	_count++;
	
}

void UserSensorsList::add2list(XBeeAddress64 addr, uint8_t type, uint8_t ch2add[], char* ds2add[], char* alarms2add[], char *fname, char *channel){
	add2list(addr, type, (uint16_t)0xFFFE, ch2add, ds2add, alarms2add, fname, channel);
}

void UserSensorsList::printlist(){
	for(uint8_t cnt=0; cnt<_count; cnt++){
		Serial.println(XBeeSensors[cnt]->addr.getMsb(), HEX);
		Serial.println(XBeeSensors[cnt]->addr.getLsb(), HEX);
		Serial.println(XBeeSensors[cnt]->type, HEX);
		Serial.println(XBeeSensors[cnt]->parent, HEX);		
	}
}

int8_t UserSensorsList::findinlist(XBeeAddress64 ADDR){
	for(uint8_t cnt=0; cnt<_count; cnt++){
		if(XBeeSensors[cnt]->addr == ADDR)
			return cnt;
	}
	return(-1);
}

void UserSensorsList::printLogTable(char value[]){
	char tmp[20];
	int n;
 	if(_count == 0)
		return;
	char* tmplistptr = value;
	for(uint8_t cnt=0; cnt<_count; cnt++){

		if(XBeeSensors[cnt]->filename != NULL){
			strncpy(tmplistptr, "<tr><td>", 8);
			tmplistptr += 8;
			
			n = sprintf(tmp, "%x", XBeeSensors[cnt]->addr.getMsb());
			strncpy(tmplistptr, tmp, n);
			tmplistptr += n;

			strncpy(tmplistptr, "</td><td>", 9);
			tmplistptr += 9;		
			
			n = sprintf(tmp, "%x", XBeeSensors[cnt]->addr.getLsb());
			strncpy(tmplistptr, tmp, n);
			tmplistptr += n;
			
			int j = strlen("</td><td><a href=\"http://192.168.1.105/");
			strncpy(tmplistptr, "</td><td><a href=\"http://192.168.1.105/", j);				// The IP is hardwired, please note that in case you change it in main file. 
			tmplistptr += j;
			
			n = sprintf(tmp, "%s", XBeeSensors[cnt]->filename);
			strncpy(tmplistptr, tmp, n);
			tmplistptr += n;
			
			j = strlen("\" target=\"_blank\">Log File</a></td></tr>");
			strncpy(tmplistptr, "\" target=\"_blank\">Log File</a></td></tr>", j);
			tmplistptr += j;	
			*tmplistptr = 0;
		}
	}
}

void UserSensorsList::printHTMLtable(char value[]){
	char tmp[20];
	int n;
	if(_count == 0)
		return;

	char* tmplistptr = value;
	for(uint8_t cnt=0; cnt<_count; cnt++){

		strncpy(tmplistptr, "<tr><td>", 8);
		tmplistptr += 8;
		
		n = sprintf(tmp, "%x", XBeeSensors[cnt]->addr.getMsb());
		strncpy(tmplistptr, tmp, n);
		tmplistptr += n;

		strncpy(tmplistptr, "</td><td>", 9);
		tmplistptr += 9;		
		
		n = sprintf(tmp, "%x", XBeeSensors[cnt]->addr.getLsb());
		strncpy(tmplistptr, tmp, n);
		tmplistptr += n;
		
		strncpy(tmplistptr, "</td><td>", 9);
		tmplistptr += 9;			
		
		switch(XBeeSensors[cnt]->type){
			case 0:
				strncpy(tmplistptr, "Coordinator", 11);
				tmplistptr += 11;
			break;
			case 1:
				strncpy(tmplistptr, "Router", 6);
				tmplistptr += 6;
			break;
			case 2:
				strncpy(tmplistptr, "EndDevice", 9);
				tmplistptr += 9;
			break;		
		}		

		strncpy(tmplistptr, "</td><td>", 9);
		tmplistptr += 9;	
		
		n = sprintf(tmp, "%x", XBeeSensors[cnt]->parent);
		strncpy(tmplistptr, tmp, n);
		tmplistptr += n;		
		
		strncpy(tmplistptr, "</td></tr>", 10);
		tmplistptr += 10;
		*tmplistptr = 0;
		// Close the first row, table should have one more row with 4 cells...
	}
}

void UserSensorsList::printTESTtable(char value[]){
	char tmp[20];
	int n;
	if(_count == 0)
		return;

	char* tmplistptr = value;
	for(uint8_t cnt=0; cnt<_count; cnt++){

		strncpy(tmplistptr, "<tr><td>", 8);
		tmplistptr += 8;
		
		n = sprintf(tmp, "%x", XBeeSensors[cnt]->addr.getMsb());
		strncpy(tmplistptr, tmp, n);
		tmplistptr += n;

		strncpy(tmplistptr, "</td><td>", 9);
		tmplistptr += 9;		
		
		n = sprintf(tmp, "%x", XBeeSensors[cnt]->addr.getLsb());
		strncpy(tmplistptr, tmp, n);
		tmplistptr += n;
		
		strncpy(tmplistptr, "</td><td>", 9);
		tmplistptr += 9;			
		
		switch(XBeeSensors[cnt]->type){
			case 0:
				strncpy(tmplistptr, "Coordinator", 11);
				tmplistptr += 11;
			break;
			case 1:
				strncpy(tmplistptr, "Router", 6);
				tmplistptr += 6;
			break;
			case 2:
				strncpy(tmplistptr, "EndDevice", 9);
				tmplistptr += 9;
			break;		
		}		
		
		strncpy(tmplistptr, "</td><td>", 9);
		tmplistptr += 9;
		
		if(XBeeSensors[cnt]->type == 2){
			n = strlen("<a id=\"e");
			strncpy(tmplistptr, "<a id=\"e", n);
			tmplistptr += n;		
			n = sprintf(tmp, "%x", XBeeSensors[cnt]->addr.getLsb());
			strncpy(tmplistptr, tmp, n);
			tmplistptr += n;
			n = strlen("\" href=\"\">Force Sample</a>");
			strncpy(tmplistptr, "\" href=\"\">Force Sample</a>", n);
			tmplistptr += n;		
		}else{
			strncpy(tmplistptr, "N/A", 3);
			tmplistptr += 3;			
		}
			
		strncpy(tmplistptr, "</td></tr>", 10);
		tmplistptr += 10;
		*tmplistptr = 0;
		// Close the first row, table should have one more row with 4 cells...
	}
}


tempC::tempC(const uint8_t c){
	ch = c;
}

uint8_t  tempC::setData(ZBRxIoSampleResponse _ioSample)
{
	if(_ioSample.isAnalogEnabled(ch)){
		Serial.println("Setting  tempC Data");
		dat = (((_ioSample.getAnalog(ch) / 1023.0) * 1200) - 500.0)/10.0;
		Serial.print("dat = ");
		Serial.println(dat, DEC);
	}else{
		Serial.print("Analog AIO(");
		Serial.print(ch, DEC);
		Serial.println(") is not enable");
		return(0);	
	}
	
}

analog::analog(const uint8_t c){
	ch = c;
}

uint8_t  analog::setData(ZBRxIoSampleResponse _ioSample)
{
	if(_ioSample.isAnalogEnabled(ch)){
		Serial.print("Setting  analog Data");
		dat = _ioSample.getAnalog(ch);
		return(1);
	}else{
		Serial.print("Analog AIO(");
		Serial.print(ch, DEC);
		Serial.println(") is not enable");
		return(0);
	}
}

onoff::onoff(const uint8_t c){
	ch = c;
}

uint8_t  onoff::setData(ZBRxIoSampleResponse _ioSample)
{
	Serial.print("Setting  onoff Data");
	if(_ioSample.isDigitalEnabled(ch)){
		dat = _ioSample.isDigitalOn(ch);	
		return(1);
	}else{
		Serial.print("Digital IO DI(");
		Serial.print(ch, DEC);
		Serial.println(") is not enable");
		return(0);
	}
}

SensorData::SensorData(uint8_t inputs[], char* ds[], char* adfs[] )
{
	if(inputs == NULL){
		for(uint8_t j=0; j < MAXSIZE; j++){
			datastream[j] = NULL;
			input[j] = NULL;
		}
		return;
	}
	for(uint8_t j=0; j < MAXSIZE; j++){
		type[j] = inputs[j];
		switch(inputs[j]){
			case(DISABLE):
				input[j] = NULL;
			break;
			case(DIGITAL):
				Serial.println("Digital");
				input[j] = (onoff*) new onoff(j);
			break;
			case(ANALOG):
				input[j] = (analog*) new analog(j);
			break;
			case(TEMPERATURE):
				input[j] =  (tempC*) new tempC(j);
			break;
		}
		if(input[j] != NULL){
			datastream[j] = (char*)malloc_check(strlen(ds[j])+1);
			if(!datastream[j]){
				Serial.println("Could not create datastream - malloc");
				datastream[j] = NULL;
			}else{
				strcpy(datastream[j], ds[j]);
				*(datastream[j]+strlen(ds[j])) = '\0';
			}
			// allocate 
			alarm[j] = adfs[j];				// Ya se hizo malloc anteriormente
/* 			alarm[j] = (char*)malloc_check(strlen(adfs[j])+1);
			if(!alarm[j]){
				Serial.println("Could not create alarm - malloc");
				alarm[j] = NULL;
			}else{
				strcpy(alarm[j], adfs[j]);
				*(alarm[j]+strlen(adfs[j])+1) = '\0';
			}	 */		
		}else{
			datastream[j] = NULL;
			alarm[j] = NULL;
		}
	}
	this->log = log;
	this->filename = filename;
	_ioSample = ZBRxIoSampleResponse();	
}

void SensorData::newSample(ZBRxIoSampleResponse SAMPLE){
	_ioSample = SAMPLE;
}

void SensorData::SetRootHandler(SdFile* root){
	this->root = root;
} 


XBeeDeviceBase::XBeeDeviceBase(XBeeAddress64  ADDR, uint16_t PARENT, uint8_t TYPE){
	addr = ADDR;
	type = TYPE;
	parent = PARENT;
	iot = 0;
	XBeeSensorPtr = NULL;
}
  
const char* XBeeDeviceBase::NodeType(){
	if(type == 0x00)
		return("Coordinator");
	else if(type == 0x01)
		return("Router");
	else
		return("End Device");
}
  
void XBeeDeviceBase::CreateSensor(UserSensorsList *usr){
	char _fs[25];
	Serial.println("Enter Create Sensor Method");
	if(type == 0x02){
		Serial.println("type = 2");
		for(int i=0; i < MAX_NUM_XBEE_SENSORS; i++){
			if(usr->XBeeSensors[i] != NULL){
				if(usr->XBeeSensors[i]->addr == addr){
					iot = 0;
					Serial.println("Sensor Data callback");
					XBeeSensorPtr = new SensorData(usr->XBeeSensors[i]->chdefs, usr->XBeeSensors[i]->dsdefs, usr->XBeeSensors[i]->alarmdefs);
					XBeeSensorPtr->root = usr->XBeeSensors[i]->root;
					XBeeSensorPtr->log = usr->XBeeSensors[i]->log;
					XBeeSensorPtr->channel = usr->XBeeSensors[i]->channel;
					XBeeSensorPtr->filename = usr->XBeeSensors[i]->filename;
					XBeeSensorPtr->csv = usr->XBeeSensors[i]->csv;
					Serial.println("ack init");
					XBeeSensorPtr->ack = 0;
					for(uint8_t k=0; k<MAXSIZE; k++){
						XBeeSensorPtr->fired[k] = 0;
					}
					Serial.println("ack init fin");
/* 					char* statPtr;
					statPtr = (char*)malloc_check(100);
					if(!statPtr){
						Serial.println("Could not allocate status memory");
						XBeeSensorPtr->status = NULL;
					}else{					
						XBeeSensorPtr->status = statPtr;
						memset(XBeeSensorPtr->status, 0, 100);
					} */
					SdFile file;
					if(XBeeSensorPtr->csv != NULL){
						Serial.println("Init csv buffer");
						memset(XBeeSensorPtr->csv, 0, CSV);
					}
					if(!file.open(XBeeSensorPtr->root, XBeeSensorPtr->filename, O_READ)){
						int j = 0;
						Serial.print("Create Filename for Sensor: ");
						Serial.println(XBeeSensorPtr->filename);
						strcat(XBeeSensorPtr->log, "Timestamp, ");
		
						for(uint8_t k=0; k<MAXSIZE; k++){
							if(XBeeSensorPtr->input[k] != NULL){
								switch(XBeeSensorPtr->type[k]){
									case(DISABLE):
										// do nothing
									break;
									case(DIGITAL):										
										if(j >= 1)
											strcat(XBeeSensorPtr->log, ", ");
										sprintf(_fs, "Digital Input %d", ((onoff*)XBeeSensorPtr->input[k])->ch);
										strcat(XBeeSensorPtr->log, _fs);
										j++;
									break;
									case(ANALOG):										
										if(j >= 1)
											strcat(XBeeSensorPtr->log, ", ");
										strcat(XBeeSensorPtr->log, "analog (counts)");
										j++;
									break;
									case(TEMPERATURE):
										if(j >= 1)
											strcat(XBeeSensorPtr->log, ", ");
										strcat(XBeeSensorPtr->log, "temperature (C)");
										j++;
									break;
								}
							}
						}
						strcat(XBeeSensorPtr->log, "\r\n");
						Serial.print(XBeeSensorPtr->log);
//						char file2open[30];
//						memset(file2open, 0, 30);
//						strcat(file2open, "/logger/");
//						strcat(file2open, XBeeSensorPtr->filename);
						if(file.open(XBeeSensorPtr->root, XBeeSensorPtr->filename, O_CREAT | O_APPEND | O_WRITE | O_SYNC)){							
							int16_t ferror = file.write(XBeeSensorPtr->log, strlen(XBeeSensorPtr->log));
							if(ferror > 0){
								file.close();
							}else{
								Serial.println("Error while writing data to file");
							}
							memset(XBeeSensorPtr->log, 0, LOG);					
							Serial.println("Create sensor Log File Succeed!");
						}else{
							Serial.println("Problem creating file!");
						}
					}else{
						Serial.print("File ");
						Serial.print(XBeeSensorPtr->filename);
						Serial.println(" alreay exists.");
						file.close();
					}
				}	   
			}	    	
		}
	}
	Serial.println("Exit Create Sensor");
}

void XBeeDeviceBase::ShowSample(ZBRxIoSampleResponse SAMPLE, String time){
	char _fs[10];
	char *_fsptr = &_fs[0];
	int j = 0;
	char t[15];
	time.toCharArray(t, time.length()+1);
	strcat(XBeeSensorPtr->log, t);
	strcat(XBeeSensorPtr->log, ", ");
	XBeeSensorPtr->newSample(SAMPLE);
	for(uint8_t k=0; k<MAXSIZE; k++){
		if(XBeeSensorPtr->input[k] != NULL){
			switch(XBeeSensorPtr->type[k]){
				case(DISABLE):
					// do nothing
				break;
				case(DIGITAL):
					if(((onoff*)XBeeSensorPtr->input[k])->setData(XBeeSensorPtr->_ioSample)){
						memset(_fs, 0, 10);
						if(j == 0){
							sprintf(_fs, "%d", ((onoff*)XBeeSensorPtr->input[k])->dat);
						}			
						if(j >= 1){
							sprintf(_fs, ", %d", ((onoff*)XBeeSensorPtr->input[k])->dat);
						}					
						j++;
						if(XBeeSensorPtr->log != NULL){
							strcat(XBeeSensorPtr->log, _fs);
						}
					}
				break;
				case(ANALOG):
					if(((analog*)XBeeSensorPtr->input[k])->setData(XBeeSensorPtr->_ioSample)){
						memset(_fs, 0, 10);
						if(j == 0){
							sprintf(_fs, "%d", ((analog*)XBeeSensorPtr->input[k])->dat);
						}
						if(j >= 1){
							sprintf(_fs, ", %d", ((analog*)XBeeSensorPtr->input[k])->dat);
						}
						j++;
						if(XBeeSensorPtr->log != NULL){
							strcat(XBeeSensorPtr->log, _fs);
						}
					}
				break;
				case(TEMPERATURE):	
					if(((tempC*)XBeeSensorPtr->input[k])->setData(XBeeSensorPtr->_ioSample)){
						memset(_fs, 0, 10);
						if(j >= 1){
							strcat(_fsptr, ", ");
							_fsptr += 2;
						}
						ftoa(_fsptr, ((tempC*)XBeeSensorPtr->input[k])->dat);
						j++;
						Serial.println("save value to log variable");
						if(XBeeSensorPtr->log != NULL){
							strcat(XBeeSensorPtr->log, _fs);
						}				
					}
				break;			
			}
			Serial.println("If datatream available");
			if(XBeeSensorPtr->datastream[k] != NULL){
				if(strlen(XBeeSensorPtr->datastream[k]) > 0){
					strcat(XBeeSensorPtr->csv, XBeeSensorPtr->datastream[k]);
					strcat(XBeeSensorPtr->csv, ",");
					strcat(XBeeSensorPtr->csv, _fs);
					strcat(XBeeSensorPtr->csv, "\n\r");
					iot = 1;
				}
			}
			Serial.println("If alarm is configured");
			if(XBeeSensorPtr->alarm[k] != NULL){
				char *ptr;
				ptr = strstr(XBeeSensorPtr->alarm[k], ":");
				if(ptr != NULL){
					switch(XBeeSensorPtr->type[k]){
						case(DISABLE):
							// do nothing
						break;
						case(DIGITAL):
							if(_fs[0] == *(ptr+1)){							// if value are the same, then proceed
								if(XBeeSensorPtr->fired[k] == 1)
									break;
								XBeeSensorPtr->fired[k] = 1;
								XBeeSensorPtr->ack = 1;
							}else{
								if(XBeeSensorPtr->fired[k] == 0)
									break;
								XBeeSensorPtr->fired[k] = 0;
								XBeeSensorPtr->ack = 1;
							}
						break;
					}
				}
			}
		}
	}	
	if(j){
		SdFile file;
		Serial.println("filename to open : ");
		Serial.println(XBeeSensorPtr->filename);
		// char file2open[30];
		// memset(file2open, 0, 30);
		// strcat(file2open, "/logger/");
		// strcat(file2open, XBeeSensorPtr->filename);		
		if(file.open(XBeeSensorPtr->root, XBeeSensorPtr->filename, O_APPEND | O_WRITE | O_SYNC)){
			Serial.println("Open File for logging: ");
			strcat(XBeeSensorPtr->log, "\r\n");
			Serial.print(XBeeSensorPtr->log);
			int16_t ferror = file.write(XBeeSensorPtr->log, strlen(XBeeSensorPtr->log));
			if(ferror > 0){
				file.close();
			}else{
				Serial.println("Error while writing data to file");
			}
			memset(XBeeSensorPtr->log, 0, LOG);						// if we handle to save the data in file just wipe the log array contents...
		}else{
			Serial.println("Could not open file for logging");
			// Wait until next time.... not very convinient but this is just a demo...
		}
	}
	// XBeeSensorPtr->saveValues();
}

int XBeeDeviceBase::BuildInternetOfThingsPUT(char** buf_ptr, const char* API_KEY_VAR){
	char* buffer = *buf_ptr;
//	String data;
	char csv_len[4];
	
	if(type == 0x02){
		if(iot == 1){
			strcpy(buffer, "PUT /v2/feeds/");
			strcat(buffer, XBeeSensorPtr->channel);
			strcat(buffer, ".csv HTTP/1.1\r\nHost: api.cosm.com\r\nX-ApiKey: ");
			strcat(buffer, API_KEY_VAR);
			strcat(buffer, "\r\nContent-Length: ");
			int len = strlen(XBeeSensorPtr->csv) - 1;
			itoa(len, csv_len, 10);
			strcat(buffer, csv_len);
			strcat(buffer, "\r\nConnection: close\r\n\r\n");
			strcat(buffer, XBeeSensorPtr->csv);
			strcat(buffer, "\r\n");
			memset(XBeeSensorPtr->csv, 0, CSV);
			iot = 0;
			return(1);
		}
	}
	return(-1);
}

uint8_t XBeeDeviceBase::StatusUpdated(void){
	return(XBeeSensorPtr->ack);
}

int XBeeDeviceBase::GetSensorStatus(char** stat_ptr, char t[]){
	char *ptr;
	char* status = *stat_ptr;
	int j = 0;
	
	if(type == 0x02){
		if(XBeeSensorPtr->ack == 1){
			for(uint8_t k=0; k<MAXSIZE; k++){
				if(XBeeSensorPtr->fired[k] == 1){
					ptr = strstr(XBeeSensorPtr->alarm[k], ":");
					strncat(status, XBeeSensorPtr->alarm[k], (ptr-XBeeSensorPtr->alarm[k]));
					strcat(status, "\n\r");
					j = 1;
				}
			}
			if(j){
				strcat(status, "@ ");
				strcat(status, t);
				strcat(status, "\n\r");
			}
			XBeeSensorPtr->ack = 0;
			return(1);
		}
	}
	return(0);
}

// EOF





