

// #ifndef Sensors_h
// #define Sensors_h

#ifndef __SENSORS_H__
#define __SENSORS_H__

#include <XBee.h>
#include <SD.h>
#include <DSPI.h>

#define MAX_NUM_XBEE_SENSORS	9
#define MAXSIZE 				13				// This is the maximum number of DIO lines that can be configure for XBee.

#define DISABLE			0 
#define DIGITAL 		1
#define ANALOG			2
#define TEMPERATURE		3

#define CSV			30
#define LOG			100

char *ftoa(char *a, double f, int precision);

class UserDefXBeeSensors
{
public:
	XBeeAddress64	addr;
	uint8_t        	type;
	uint16_t       	parent;
	uint8_t		   	chdefs[MAXSIZE];
	char*			dsdefs[MAXSIZE];
	char*			alarmdefs[MAXSIZE];
	char*		   	filename;
	char*		   	log;
	char*			csv;
	char*			channel;
	SdFile* 	   	root;
};
 
struct tempC{
	static const uint8_t type = 3;
	uint8_t ch;				// XBee Analog channel
	float dat;				// Temperature Data
	tempC(const uint8_t c);
	uint8_t  setData(ZBRxIoSampleResponse _ioSample);
};

struct analog{
	static const uint8_t type = 2;
	uint8_t ch;					// XBee Analog channel
	uint16_t dat;				// Analog Data
	analog(const uint8_t c);
	uint8_t  setData(ZBRxIoSampleResponse _ioSample);
};

struct onoff{
	static const uint8_t type = 1;
	uint8_t ch;				// XBee Analog channel
	bool dat;				// Temperature Data
	onoff(const uint8_t ch);
	uint8_t  setData(ZBRxIoSampleResponse _ioSample);
};
 
class SensorData
{
public:
	char*		channel;		
	char*		status;
	uint8_t		ack;
	uint8_t		fired[MAXSIZE];
	char* 		alarm[MAXSIZE];
	char* 		datastream[MAXSIZE];	
	uint8_t		type[MAXSIZE];
	void*		input[MAXSIZE];
	char*  	 	log;
	char*		csv;
	char*  		filename;
	SdFile*		root;	
	ZBRxIoSampleResponse _ioSample;            			// this will hold the last sample acquired	
public:
	SensorData(uint8_t inputs[], char* ds[], char* adfs[]);
	void  	newSample(ZBRxIoSampleResponse SAMPLE);
	void 	SetRootHandler(SdFile* root);
};
 
class UserSensorsList
{
public:
	SdFile*		root;
	UserDefXBeeSensors* XBeeSensors[MAX_NUM_XBEE_SENSORS]; 
private:
	uint8_t	_count;
public:
	UserSensorsList();				// User Sensor Object List constructor
//	~UserSensorsList(){} ; 			// Destructor
	void add2list(XBeeAddress64 addr, uint8_t type, uint16_t parent, uint8_t ch2add[], char* ds2add[], char* alarms2add[], char *fname, char *channel);
	void add2list(XBeeAddress64 addr, uint8_t type, uint8_t ch2add[], char* ds2add[], char* alarms2add[], char *fname, char *channel);
	void printlist();
	int8_t findinlist(XBeeAddress64 ADDR);
	void printHTMLtable(char value[]);
	void printLogTable(char value[]);
	void printTESTtable(char value[]);
	void SetRootHandler(SdFile* root);
};
  
  
class XBeeDeviceBase{
  
public:  
	SensorData*  	XBeeSensorPtr;
	XBeeAddress64  	addr;
	uint8_t        	type;
	uint16_t       	parent;
	uint8_t			iot;
public:
	XBeeDeviceBase(XBeeAddress64  ADDR, uint16_t PARENT, uint8_t TYPE);
	const char* NodeType();
	void CreateSensor(UserSensorsList *usr);
	void ShowSample(ZBRxIoSampleResponse SAMPLE, String time);
	int BuildInternetOfThingsPUT(char** buf_ptr, const char* API_KEY_VAR);
	uint8_t StatusUpdated(void);
	int GetSensorStatus(char** stat_ptr, char t[]);
};

#endif
